@@ -1,5 +1,5 @@
 /***************************************
-  $Header: /cvsroot/petscgraphics/chts.c,v 1.28 2004/06/30 15:38:17 hazelsct Exp $
+  $Header: /cvsroot/petscgraphics/chts.c,v 1.35 2006/01/30 04:53:58 hazelsct Exp $
 
   This is the Cahn Hilliard timestepping code.  It is provided here as an
   example usage of the Illuminator Distributed Visualization Library.
@@ -32,6 +32,7 @@ Parallelism can be invoked based on the DA construct:\n\
 
 
 /* Including cahnhill.h includes the necessary PETSc header files */
+#include <stdlib.h>
 #include "cahnhill.h"
 #include "illuminator.h"
 
@@ -50,6 +51,8 @@ int FormInitialCondition(AppCtx*,Vec);
 int UserLevelEnd(AppCtx*,Vec);
 int InitializeProblem(AppCtx*,Vec*);
 
+ISurface Surf;
+IDisplay Disp;
 
 /*++++++++++++++++++++++++++++++++++++++
   Wrapper for
@@ -197,13 +200,19 @@ int ch_ts_monitor (TS thets, int stepno, PetscScalar time, Vec X, void *ptr)
     {
       if (user->threedee)
 	{
+#ifdef GEOMVIEW
 	  DPRINTF ("Starting triangulation\n",0);
-	  ierr = DATriangulate (user->da, X, user->chvar, minmax, PETSC_DECIDE,
-				PETSC_NULL, PETSC_NULL, PETSC_FALSE,
-				PETSC_FALSE, PETSC_FALSE); CHKERRQ (ierr);
+	  ierr = DATriangulate (Surf, user->da, X, user->chvar, minmax,
+				PETSC_DECIDE, PETSC_NULL, PETSC_NULL,
+				PETSC_FALSE, PETSC_FALSE, PETSC_FALSE);
+	  CHKERRQ (ierr);
 	  DPRINTF ("Done, sending to Geomview\n",0);
 	  ierr = GeomviewDisplayTriangulation
-	    (user->comm, minmax, "Cahn-Hilliard", PETSC_TRUE); CHKERRQ (ierr);
+	    (user->comm, Surf, Disp, minmax, "Cahn-Hilliard", PETSC_TRUE);
+	  CHKERRQ (ierr);
+#endif
+
+	  ierr = SurfClear (Surf); CHKERRQ (ierr);
 	  DPRINTF ("Done.\n",0);
 	}
       else
@@ -228,8 +237,10 @@ int ch_ts_monitor (TS thets, int stepno, PetscScalar time, Vec X, void *ptr)
       snprintf (paramdata[2], 49, "%g", user->gamma);
       snprintf (paramdata[3], 49, "%g", user->kappa);
       DPRINTF ("Storing data using IlluMultiSave()\n",0);
-      IlluMultiSave (user->da, X, filename, 1.,1.,1., &chtype, 4, paramnames,
-		     paramdata, COMPRESS_INT_NONE | COMPRESS_GZIP_FAST);
+      ierr = IlluMultiSave
+	(PETSC_COMM_WORLD, user->da, X, filename, 1.,1.,1., &chtype, 4,
+	 paramnames, paramdata, COMPRESS_INT_NONE | COMPRESS_GZIP_FAST);
+      CHKERRQ (ierr);
     }
 
   DPRINTF ("Completed timestep monitor tasks.\n",0);
@@ -285,7 +296,10 @@ int main (int argc, char **argv)
     {
       if (user.threedee)
 	{
-	  ierr = GeomviewBegin (PETSC_COMM_WORLD); CHKERRQ (ierr);
+	  ierr = SurfCreate (&Surf); CHKERRQ (ierr);
+#ifdef GEOMVIEW
+	  ierr = GeomviewBegin (PETSC_COMM_WORLD, &Disp); CHKERRQ (ierr);
+#endif
 	}
       else
 	{
@@ -356,7 +370,10 @@ int main (int argc, char **argv)
     {
       if (user.threedee)
 	{
-	  ierr = GeomviewEnd (PETSC_COMM_WORLD); CHKERRQ (ierr);
+#ifdef GEOMVIEW
+	  ierr = GeomviewEnd (PETSC_COMM_WORLD, Disp); CHKERRQ (ierr);
+#endif
+	  ierr = ISurfDestroy (Surf); CHKERRQ (ierr);
 	}
       else
 	{
@@ -414,7 +431,7 @@ int FormInitialCondition (AppCtx *user, Vec X)
       ierr = PetscRandomCreate (user->comm, RANDOM_DEFAULT, &rand);
       CHKERRQ (ierr);
       ierr = PetscRandomSetInterval (rand, 0.35, 0.65); CHKERRQ (ierr);
-      ierr = VecSetRandom (rand, X); CHKERRQ (ierr);
+      ierr = VecSetRandom (X, rand); CHKERRQ (ierr);
       ierr = PetscRandomDestroy (rand); CHKERRQ (ierr);
     }
   else if (user->load_data > -1)
@@ -424,8 +441,8 @@ int FormInitialCondition (AppCtx *user, Vec X)
       sprintf (basename, "chtsout.time%.3d", user->load_data);
       DPRINTF ("Loading data for timestep %d, basename %s\n", user->load_data,
 	       basename);
-      IlluMultiRead (user->da, X, basename, &usermetacount, &usermetanames,
-		     &usermetadata);
+      IlluMultiRead (PETSC_COMM_WORLD, user->da, X, basename, &usermetacount,
+		     &usermetanames, &usermetadata);
       /* TODO: free these */
       for (i=0; i<usermetacount; i++)
 	PetscPrintf (PETSC_COMM_WORLD, "Parameter \"%s\" = \"%s\"\n",
