@@ -1,5 +1,5 @@
 /***************************************
-  $Header: /cvsroot/petscgraphics/geomview.c,v 1.6 2003/04/30 19:34:55 hazelsct Exp $
+  $Header: /cvsroot/petscgraphics/geomview.c,v 1.14 2006/02/05 21:15:52 hazelsct Exp $
 
   This file has the Geomview interface, including the PETSc vector gather
   operations to bring everything to CPU 0.
@@ -8,8 +8,11 @@
 
 #include <stdio.h>
 #include <petscvec.h>
-#include "config.h" /* esp. for inline */
+#include "config.h"      /* esp. for inline */
 #include "illuminator.h" /* Just to make sure the interface is "right" */
+#include "surface.h"     /* For isurface structure definition */
+#include <stdlib.h>      /* For malloc() */
+#include <unistd.h>      /* For execlp() */
 
 /* Build with -DDEBUG for debugging output */
 #undef DPRINTF
@@ -19,19 +22,6 @@
 #define DPRINTF(fmt, args...)
 #endif
 
-/*+ Stream holding the pipe to the Geomview process.  It is initialized to
-  NULL, then set by GeomviewBegin, and cleared by GeomviewEnd.  This can be
-  used as a flag to determine whether the Geomview display is open. +*/
-static FILE *pfd = NULL;
-
-/*+ Declared in illuminator.c, this gives the current number of triangles on
-  this node. +*/
-extern int num_triangles;
-
-/*+ Declared in illuminator.c, this array stores the corner coordinates and
-  color information for each triangle. +*/
-extern PetscScalar vertices[];
-
 
 #undef __FUNCT__
 #define __FUNCT__ "GeomviewBegin"
@@ -44,22 +34,32 @@ extern PetscScalar vertices[];
 
   MPI_Comm comm MPI communicator for rank information, if NULL it uses
   PETSC_COMM_WORLD.
+
+  IDisplay *newdisp Pointer in which to put a new IDisplay object.
   ++++++++++++++++++++++++++++++++++++++*/
 
-int GeomviewBegin(MPI_Comm comm)
+int GeomviewBegin(MPI_Comm comm, IDisplay *newdisp)
 {
   int rank, ierr, gv_pid,gv_pipe[2], to_gv_pipe[2];
   char gv_version[100];
+  struct idisplay *thedisp;
+
+  if (!(thedisp = (struct idisplay *) malloc (sizeof (struct idisplay))))
+    SETERRQ (PETSC_ERR_MEM, "Unable to allocate memory for isurface object");
+
+  /* Initialize display image to zero */
+  thedisp->rgb = NULL;
+  thedisp->rgb_width = thedisp->rgb_height = thedisp->rgb_rowskip =
+    thedisp->rgb_bpp = 0;
+  thedisp->zbuf = NULL;
+  thedisp->zbuf_width = thedisp->zbuf_height = thedisp->zbuf_depth =
+    thedisp->zbuf_rowskip = 0;
 
   if (!comm)
     comm = PETSC_COMM_WORLD;
   ierr = MPI_Comm_rank (comm,&rank); CHKERRQ(ierr);
   if (!rank)
     {
-      if (pfd)
-	SETERRQ (PETSC_ERR_ARG_WRONGSTATE,
-		 "Multiple geomview displays not allowed");
-
       pipe (gv_pipe); /* from geomview stdout */
       pipe (to_gv_pipe); /* to geomview stdin */
       gv_pid = fork ();
@@ -83,11 +83,13 @@ int GeomviewBegin(MPI_Comm comm)
       /* PETSc program execution resumes here */
       close (gv_pipe[1]);
       close (to_gv_pipe[0]);
-      pfd = fdopen (to_gv_pipe[1], "w"); /* hooked to stdin of geomview */
-      fprintf (pfd, "(echo (geomview-version) \"\n\")\n");
-      fflush (pfd);
+      thedisp->to_geomview = fdopen (to_gv_pipe[1], "w"); /* geomview stdin */
+      fprintf (thedisp->to_geomview, "(echo (geomview-version) \"\n\")\n");
+      fflush (thedisp->to_geomview);
       read (gv_pipe[0], gv_version, sizeof (gv_version));
     }
+
+  *newdisp = (IDisplay) thedisp;
   return 0;
 }
 
@@ -103,22 +105,25 @@ int GeomviewBegin(MPI_Comm comm)
 
   MPI_Comm comm MPI communicator for rank information, if NULL it uses
   PETSC_COMM_WORLD.
+
+  IDisplay Disp IDisplay object whose geomview FILE we're closing.
   ++++++++++++++++++++++++++++++++++++++*/
 
-int GeomviewEnd (MPI_Comm comm)
+int GeomviewEnd (MPI_Comm comm, IDisplay Disp)
 {
   int rank, ierr;
+  struct idisplay *thedisp = (struct idisplay *) Disp;
 
   if (!comm)
     comm = PETSC_COMM_WORLD;
   ierr = MPI_Comm_rank (comm, &rank); CHKERRQ (ierr);
   if (!rank)
     {
-      if (pfd)
+      if (thedisp->to_geomview)
 	{
-	  fprintf (pfd, "(exit)");
-	  fclose (pfd);
-	  pfd = NULL;
+	  fprintf (thedisp->to_geomview, "(exit)");
+	  fclose (thedisp->to_geomview);
+	  thedisp->to_geomview = NULL;
 	}
     }
   return 0;
@@ -137,6 +142,10 @@ int GeomviewEnd (MPI_Comm comm)
   MPI_Comm comm MPI communicator for rank information, if NULL it uses
   PETSC_COMM_WORLD.
 
+  ISurface Surf ISurface object containing triangles to render using Geomview.
+
+  IDisplay Disp IDisplay object whose geomview FILE we're closing.
+
   PetscScalar *minmax Position of block corners: xmin, xmax, ymin, ymax, zmin, zmax.
 
   char *name Name to give the Geomview OOGL object which we create.
@@ -145,25 +154,29 @@ int GeomviewEnd (MPI_Comm comm)
   ++++++++++++++++++++++++++++++++++++++*/
 
 int GeomviewDisplayTriangulation
-(MPI_Comm comm, PetscScalar *minmax, char *name, PetscTruth transparent)
+(MPI_Comm comm, ISurface Surf, IDisplay Disp, PetscScalar *minmax, char *name,
+ PetscTruth transparent)
 {
   int ierr, i, total_entries, start, end, rank;
   PetscScalar *localvals;
   Vec globalverts, localverts;
   VecScatter vecscat;
   IS islocal;
+  struct isurface *thesurf = (struct isurface *) Surf;
+  struct idisplay *thedisp = (struct idisplay *) Disp;
 
   /* Simple "assertion" check */
   if (!comm)
     comm = PETSC_COMM_WORLD;
   ierr = MPI_Comm_rank (comm, &rank); CHKERRQ (ierr);
-  if (!rank && !pfd)
+  if (!rank && !thedisp->to_geomview)
     SETERRQ (PETSC_ERR_ARG_WRONGSTATE,"Geomview display is not open!");
 
   /*+ First, this creates global and local vectors for all of the triangle
     vertices. +*/
-  ierr = VecCreateMPIWithArray (comm, 13*num_triangles, PETSC_DETERMINE,
-				vertices, &globalverts); CHKERRQ (ierr);
+  ierr = VecCreateMPIWithArray (comm, 13*thesurf->num_triangles,
+				PETSC_DETERMINE,
+				thesurf->vertices, &globalverts); CHKERRQ (ierr);
   ierr = VecGetSize (globalverts, &total_entries); CHKERRQ (ierr);
   ierr = VecCreateMPI (comm, (rank == 0) ? total_entries:0, total_entries,
 		       &localverts); CHKERRQ (ierr);
@@ -203,29 +216,31 @@ int GeomviewDisplayTriangulation
 
   /*+ Finally, it sends everything to Geomview, +*/
   if (!rank) {
-    fprintf (pfd, "(geometry \"%s\" { : bor })", name);
-    fprintf (pfd, "(read geometry { define bor \n");
-    fprintf (pfd, "appearance {%s}\nOFF\n", transparent ? "+transparent" : "");
-    fprintf (pfd, "%d %d 0\n", 3*(total_entries/13) + 2, total_entries/13);
+    fprintf (thedisp->to_geomview, "(geometry \"%s\" { : bor })", name);
+    fprintf (thedisp->to_geomview, "(read geometry { define bor \n");
+    fprintf (thedisp->to_geomview, "appearance {%s}\nOFF\n",
+	     transparent ? "+transparent" : "");
+    fprintf (thedisp->to_geomview, "%d %d 0\n", 3*(total_entries/13) + 2,
+	     total_entries/13);
     for (i=0; i<total_entries; i+=13)
-      fprintf (pfd, "%g %g %g\n%g %g %g\n%g %g %g\n", localvals[i],
-	       localvals[i+1], localvals[i+2], localvals[i+3], localvals[i+4],
-	       localvals[i+5], localvals[i+6], localvals[i+7], localvals[i+8]);
-    fprintf (pfd, "%g %g %g\n%g %g %g\n", minmax[0], minmax[2], minmax[4],
-	     minmax[1], minmax[3], minmax[5]);
+      fprintf (thedisp->to_geomview, "%g %g %g\n%g %g %g\n%g %g %g\n",
+	       localvals[i], localvals[i+1], localvals[i+2], localvals[i+3],
+	       localvals[i+4], localvals[i+5], localvals[i+6], localvals[i+7],
+	       localvals[i+8]);
+    fprintf (thedisp->to_geomview, "%g %g %g\n%g %g %g\n", minmax[0],minmax[2],
+	     minmax[4], minmax[1], minmax[3], minmax[5]);
     for (i=0; i<total_entries/13; i++)
-      fprintf (pfd, "3 %d %d %d %g %g %g %g\n", 3*i,3*i+1,3*i+2,
+      fprintf (thedisp->to_geomview,"3 %d %d %d %g %g %g %g\n",3*i,3*i+1,3*i+2,
 	       localvals[13*i+9], localvals[13*i+10], localvals[13*i+11],
 	       localvals[13*i+12]);
-    fprintf (pfd, "})\n");
-    fflush (pfd);
+    fprintf (thedisp->to_geomview, "})\n");
+    fflush (thedisp->to_geomview);
   }
 
-  /*+ and cleans up the mess, resetting the number of triangles to zero. +*/
+  /*+ and cleans up the mess. +*/
   ierr = VecRestoreArray (localverts, &localvals); CHKERRQ (ierr);
   ierr = VecDestroy (localverts); CHKERRQ (ierr);
   ierr = VecDestroy (globalverts); CHKERRQ (ierr);
-  num_triangles = 0;
 
   return 0;
 }
