@@ -1,34 +1,99 @@
 /***************************************
-  $Header: /cvsroot/petscgraphics/illuminator.c,v 1.9 2004/08/17 15:05:56 hazelsct Exp $
+  $Header: /cvsroot/petscgraphics/illuminator.c,v 1.17 2005/11/17 23:11:47 hazelsct Exp $
 
   This is the illuminator.c main file.  It has all of the routines which
   compute the triangulation in a distributed way.
 ***************************************/
 
 
-#include "config.h" /* esp. for inline */
+#include "config.h"      /* esp. for inline */
 #include "illuminator.h" /* Just to make sure the interface is "right" */
+#include "surface.h"     /* For the isurface structure definition */
+#include <stdlib.h>      /* For malloc() */
 
-#define MAX_TRIANGLES 2000000 /*+ Maximum number of generated triangles per
-			      node. +*/
+/* Build with -DDEBUG for debugging output */
+#undef DPRINTF
+#ifdef DEBUG
+#define DPRINTF(fmt, args...) PetscPrintf (PETSC_COMM_WORLD, "%s: " fmt, __FUNCT__, args)
+#else
+#define DPRINTF(fmt, args...)
+#endif
 
-/*+ Number of triangles in this node.  Its value is initialized to zero, and
-  incremented as each triangle is added, then reset to zero when the
-  triangulation is displayed. +*/
-int num_triangles=0; 
+#define MAX_TRIANGLES 10000000 /*+ Maximum number of generated triangles per
+				node. +*/
 
-/*+ Array of vertex corners, whose size is fixed by MAX_TRIANGLES.  For each
-  triangle, this array has the coordinates of the three nodes, and its R, G, B
-  and A color values, hence 13 PetscScalars for each triangle. +*/
-PetscScalar vertices[13*MAX_TRIANGLES];
+
+#undef __FUNCT__
+#define __FUNCT__ "ISurfCreate"
+
+/*++++++++++++++++++++++++++++++++++++++
+  Constructor for ISurface data object.
+
+  int SurfCreate Returns zero or an error code.
+
+  Surf *newsurf Address in which to put the new ISurface object.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+int SurfCreate (ISurface *newsurf)
+{
+  struct isurface *thesurf;
+
+  if (!(thesurf = (struct isurface *) malloc (sizeof (struct isurface))))
+    SETERRQ (PETSC_ERR_MEM, "Unable to allocate memory for isurface object");
+
+  thesurf->vertices = NULL;
+  thesurf->vertisize = thesurf->num_triangles = 0;
+  *newsurf = (ISurface) thesurf;
+  return 0;
+}
+
+
+#undef __FUNCT__
+#define __FUNCT__ "ISurfDestroy"
+
+/*++++++++++++++++++++++++++++++++++++++
+  Destructor for ISurface data object.
+
+  int ISurfDestroy Returns zero or an error code.
+
+  ISurface Surf ISurface object to destroy.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+int ISurfDestroy (ISurface Surf)
+{
+  struct isurface *thesurf = (struct isurface *) Surf;
+
+  if (thesurf->vertices)
+    free (thesurf->vertices);
+  free (thesurf);
+  return 0;
+}
+
+
+#undef __FUNCT__
+#define __FUNCT__ "ISurfClear"
+
+/*++++++++++++++++++++++++++++++++++++++
+  Clear out an isurface data object without freeing its memory.
+
+  int SurfClear Returns zero or an error code.
+
+  ISurface Surf ISurface object to clear out.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+int SurfClear (ISurface Surf)
+{
+  struct isurface *thesurf = (struct isurface *) Surf;
+
+  return thesurf->num_triangles = 0;
+}
 
 
 #undef __FUNCT__
 #define __FUNCT__ "storetri"
 
 /*++++++++++++++++++++++++++++++++++++++
-  This little inline routine just implements triangle storage.  Maybe it will
-  be more sophisticated in the future.
+  This little inline routine just implements triangle storage.
 
   int storetri Returns 0 or an error code.
 
@@ -51,29 +116,52 @@ PetscScalar vertices[13*MAX_TRIANGLES];
   PetscScalar z2 Z-coordinate of corner 2.
 
   PetscScalar *color R,G,B,A quad for this triangle.
+
+  struct isurface *thesurf ISurface object to put the triangles in.
   ++++++++++++++++++++++++++++++++++++++*/
 
 static inline int storetri
-(PetscScalar x0,PetscScalar y0,PetscScalar z0,PetscScalar x1,PetscScalar y1,
- PetscScalar z1,PetscScalar x2,PetscScalar y2,PetscScalar z2,
- PetscScalar *color) 
+(PetscScalar x0,PetscScalar y0,PetscScalar z0,
+ PetscScalar x1,PetscScalar y1,PetscScalar z1,
+ PetscScalar x2,PetscScalar y2,PetscScalar z2, PetscScalar *color,
+ struct isurface *thesurf) 
 {
-  vertices[13*num_triangles+0] = x0;
-  vertices[13*num_triangles+1] = y0;
-  vertices[13*num_triangles+2] = z0;
-  vertices[13*num_triangles+3] = x1;
-  vertices[13*num_triangles+4] = y1;
-  vertices[13*num_triangles+5] = z1;
-  vertices[13*num_triangles+6] = x2;
-  vertices[13*num_triangles+7] = y2;
-  vertices[13*num_triangles+8] = z2;
-  vertices[13*num_triangles+9] = color[0];
-  vertices[13*num_triangles+10] = color[1];
-  vertices[13*num_triangles+11] = color[2];
-  vertices[13*num_triangles+12] = color[3];
+  if (thesurf->num_triangles >= thesurf->vertisize)
+    {
+      thesurf->vertices = (PetscScalar *) realloc
+	(thesurf->vertices, 13*sizeof(double)*
+	 (thesurf->vertisize = (thesurf->vertisize==0) ? 1000 :
+	  ((thesurf->vertisize > MAX_TRIANGLES) ?
+	   thesurf->vertisize : thesurf->vertisize*2)));
+      if (thesurf->num_triangles >= thesurf->vertisize)
+	{
+	  int rank, ierr;
+	  ierr = MPI_Comm_rank (PETSC_COMM_WORLD, &rank); CHKERRQ (ierr);
+	  printf ("CPU %d: Number of triangles exceeded %d, emptying array.\n"
+		  "(The limit is MAX_TRIANGLES in illuminator.c line 21.)",
+		  rank, MAX_TRIANGLES);
+	  thesurf->num_triangles = 0;
+	}
+      else
+	DPRINTF ("Expanding vertices array to %d triangles, pointer at 0x%lx\n",
+		 thesurf->vertisize, thesurf->vertices);
+    }
 
-  if (++num_triangles >= MAX_TRIANGLES)
-    return --num_triangles;
+  thesurf->vertices[13*thesurf->num_triangles+0] = x0;
+  thesurf->vertices[13*thesurf->num_triangles+1] = y0;
+  thesurf->vertices[13*thesurf->num_triangles+2] = z0;
+  thesurf->vertices[13*thesurf->num_triangles+3] = x1;
+  thesurf->vertices[13*thesurf->num_triangles+4] = y1;
+  thesurf->vertices[13*thesurf->num_triangles+5] = z1;
+  thesurf->vertices[13*thesurf->num_triangles+6] = x2;
+  thesurf->vertices[13*thesurf->num_triangles+7] = y2;
+  thesurf->vertices[13*thesurf->num_triangles+8] = z2;
+  thesurf->vertices[13*thesurf->num_triangles+9] = color[0];
+  thesurf->vertices[13*thesurf->num_triangles+10] = color[1];
+  thesurf->vertices[13*thesurf->num_triangles+11] = color[2];
+  thesurf->vertices[13*thesurf->num_triangles+12] = color[3];
+
+  thesurf->num_triangles++;
   return 0;
 }
 
@@ -136,6 +224,8 @@ static inline int storetri
   int whichplane Index of which edge intercept(s) is between zero and 1.
 
   PetscScalar *color R,G,B,A quad for this tetrahedron.
+
+  struct isurface *thesurf ISurface object to put the triangles in.
   ++++++++++++++++++++++++++++++++++++++*/
 
 static inline int DrawTetWithPlane
@@ -144,7 +234,7 @@ static inline int DrawTetWithPlane
  PetscScalar x2,PetscScalar y2,PetscScalar z2,PetscScalar f2,
  PetscScalar x3,PetscScalar y3,PetscScalar z3,PetscScalar f3,
  PetscScalar isoquant, PetscScalar edge0,PetscScalar edge1,PetscScalar edge3,
- int whichplane, PetscScalar *color)
+ int whichplane, PetscScalar *color, struct isurface *thesurf)
 {
   PetscScalar edge2,edge4,edge5;
 
@@ -157,21 +247,21 @@ static inline int DrawTetWithPlane
       storetri (COORD(x0,x1,edge0), COORD(y0,y1,edge0), COORD(z0,z1,edge0),
 		COORD(x1,x2,edge1), COORD(y1,y2,edge1), COORD(z1,z2,edge1),
 		COORD(x0,x3,edge3), COORD(y0,y3,edge3), COORD(z0,z3,edge3),
-		color);
+		color, thesurf);
       storetri (COORD(x0,x3,edge3), COORD(y0,y3,edge3), COORD(z0,z3,edge3),
 		COORD(x1,x2,edge1), COORD(y1,y2,edge1), COORD(z1,z2,edge1),
 		COORD(x2,x3,edge5), COORD(y2,y3,edge5), COORD(z2,z3,edge5),
-		color);
+		color, thesurf);
     }
     else {
       storetri (COORD(x0,x1,edge0), COORD(y0,y1,edge0), COORD(z0,z1,edge0),
 		COORD(x0,x3,edge3), COORD(y0,y3,edge3), COORD(z0,z3,edge3),
 		COORD(x1,x2,edge1), COORD(y1,y2,edge1), COORD(z1,z2,edge1),
-		color);
+		color, thesurf);
       storetri (COORD(x1,x2,edge1), COORD(y1,y2,edge1), COORD(z1,z2,edge1),
 		COORD(x0,x3,edge3), COORD(y0,y3,edge3), COORD(z0,z3,edge3),
 		COORD(x2,x3,edge5), COORD(y2,y3,edge5), COORD(z2,z3,edge5),
-		color);
+		color, thesurf);
     }
     break;
   }
@@ -185,21 +275,21 @@ static inline int DrawTetWithPlane
       storetri (COORD(x1,x2,edge1), COORD(y1,y2,edge1), COORD(z1,z2,edge1),
 		COORD(x2,x0,edge2), COORD(y2,y0,edge2), COORD(z2,z0,edge2),
 		COORD(x1,x3,edge4), COORD(y1,y3,edge4), COORD(z1,z3,edge4),
-		color);
+		color, thesurf);
       storetri (COORD(x1,x3,edge4), COORD(y1,y3,edge4), COORD(z1,z3,edge4),
 		COORD(x2,x0,edge2), COORD(y2,y0,edge2), COORD(z2,z0,edge2),
 		COORD(x0,x3,edge3), COORD(y0,y3,edge3), COORD(z0,z3,edge3),
-		color);
+		color, thesurf);
     }
     else {
       storetri (COORD(x1,x2,edge1), COORD(y1,y2,edge1), COORD(z1,z2,edge1),
 		COORD(x1,x3,edge4), COORD(y1,y3,edge4), COORD(z1,z3,edge4),
 		COORD(x2,x0,edge2), COORD(y2,y0,edge2), COORD(z2,z0,edge2),
-		color);
+		color, thesurf);
       storetri (COORD(x2,x0,edge2), COORD(y2,y0,edge2), COORD(z2,z0,edge2),
 		COORD(x1,x3,edge4), COORD(y1,y3,edge4), COORD(z1,z3,edge4),
 		COORD(x0,x3,edge3), COORD(y0,y3,edge3), COORD(z0,z3,edge3),
-		color);
+		color, thesurf);
     }
     break;
   }
@@ -212,12 +302,12 @@ static inline int DrawTetWithPlane
       storetri (COORD(x0,x1,edge0), COORD(y0,y1,edge0), COORD(z0,z1,edge0),
 		COORD(x2,x0,edge2), COORD(y2,y0,edge2), COORD(z2,z0,edge2),
 		COORD(x0,x3,edge3), COORD(y0,y3,edge3), COORD(z0,z3,edge3),
-		color);
+		color, thesurf);
     else
       storetri (COORD(x0,x1,edge0), COORD(y0,y1,edge0), COORD(z0,z1,edge0),
 		COORD(x0,x3,edge3), COORD(y0,y3,edge3), COORD(z0,z3,edge3),
 		COORD(x2,x0,edge2), COORD(y2,y0,edge2), COORD(z2,z0,edge2),
-		color);
+		color, thesurf);
     break;
   }
 
@@ -230,12 +320,12 @@ static inline int DrawTetWithPlane
       storetri (COORD(x0,x3,edge3), COORD(y0,y3,edge3), COORD(z0,z3,edge3),
 		COORD(x1,x3,edge4), COORD(y1,y3,edge4), COORD(z1,z3,edge4),
 		COORD(x2,x3,edge5), COORD(y2,y3,edge5), COORD(z2,z3,edge5),
-		color);
+		color, thesurf);
     else
       storetri (COORD(x0,x3,edge3), COORD(y0,y3,edge3), COORD(z0,z3,edge3),
 		COORD(x2,x3,edge5), COORD(y2,y3,edge5), COORD(z2,z3,edge5),
 		COORD(x1,x3,edge4), COORD(y1,y3,edge4), COORD(z1,z3,edge4),
-		color);
+		color, thesurf);
     break;
   }
 
@@ -247,12 +337,12 @@ static inline int DrawTetWithPlane
       storetri (COORD(x0,x1,edge0), COORD(y0,y1,edge0), COORD(z0,z1,edge0),
 		COORD(x1,x2,edge1), COORD(y1,y2,edge1), COORD(z1,z2,edge1),
 		COORD(x1,x3,edge4), COORD(y1,y3,edge4), COORD(z1,z3,edge4),
-		color);
+		color, thesurf);
     else
       storetri (COORD(x0,x1,edge0), COORD(y0,y1,edge0), COORD(z0,z1,edge0),
 		COORD(x1,x3,edge4), COORD(y1,y3,edge4), COORD(z1,z3,edge4),
 		COORD(x1,x2,edge1), COORD(y1,y2,edge1), COORD(z1,z2,edge1),
-		color);
+		color, thesurf);
     break;
   }
 
@@ -265,12 +355,12 @@ static inline int DrawTetWithPlane
       storetri (COORD(x1,x2,edge1), COORD(y1,y2,edge1), COORD(z1,z2,edge1),
 		COORD(x2,x0,edge2), COORD(y2,y0,edge2), COORD(z2,z0,edge2),
 		COORD(x2,x3,edge5), COORD(y2,y3,edge5), COORD(z2,z3,edge5),
-		color);
+		color, thesurf);
     else
       storetri (COORD(x1,x2,edge1), COORD(y1,y2,edge1), COORD(z1,z2,edge1),
 		COORD(x2,x3,edge5), COORD(y2,y3,edge5), COORD(z2,z3,edge5),
 		COORD(x2,x0,edge2), COORD(y2,y0,edge2), COORD(z2,z0,edge2),
-		color);
+		color, thesurf);
     break;
   }
 
@@ -284,21 +374,21 @@ static inline int DrawTetWithPlane
       storetri (COORD(x0,x1,edge0), COORD(y0,y1,edge0), COORD(z0,z1,edge0),
 		COORD(x2,x0,edge2), COORD(y2,y0,edge2), COORD(z2,z0,edge2),
 		COORD(x1,x3,edge4), COORD(y1,y3,edge4), COORD(z1,z3,edge4),
-		color);
+		color, thesurf);
       storetri (COORD(x1,x3,edge4), COORD(y1,y3,edge4), COORD(z1,z3,edge4),
 		COORD(x2,x0,edge2), COORD(y2,y0,edge2), COORD(z2,z0,edge2),
 		COORD(x2,x3,edge5), COORD(y2,y3,edge5), COORD(z2,z3,edge5),
-		color);
+		color, thesurf);
     }
     else {
       storetri (COORD(x0,x1,edge0), COORD(y0,y1,edge0), COORD(z0,z1,edge0),
 		COORD(x1,x3,edge4), COORD(y1,y3,edge4), COORD(z1,z3,edge4),
 		COORD(x2,x0,edge2), COORD(y2,y0,edge2), COORD(z2,z0,edge2),
-		color);
+		color, thesurf);
       storetri (COORD(x2,x0,edge2), COORD(y2,y0,edge2), COORD(z2,z0,edge2),
 		COORD(x1,x3,edge4), COORD(y1,y3,edge4), COORD(z1,z3,edge4),
 		COORD(x2,x3,edge5), COORD(y2,y3,edge5), COORD(z2,z3,edge5),
-		color);
+		color, thesurf);
     }
     break;
   }
@@ -319,7 +409,10 @@ static inline int DrawTetWithPlane
 
   int DrawTet Returns 0 or an error code.
 
-  PetscScalar *coords Coordinates of tetrahedron corner points: x0, y0, z0, x1, etc.
+  ISurface Surf ISurface object into which to draw this tetrahedron's isoquant.
+
+  PetscScalar *coords Coordinates of tetrahedron corner points: x0, y0, z0, x1,
+  etc.
 
   PetscScalar *vals Function values at tetrahedron corners: f0, f1, f2, f3.
 
@@ -329,11 +422,12 @@ static inline int DrawTetWithPlane
   ++++++++++++++++++++++++++++++++++++++*/
 
 int DrawTet
-(PetscScalar *coords, PetscScalar *vals, PetscScalar isoquant,
+(ISurface Surf, PetscScalar *coords, PetscScalar *vals, PetscScalar isoquant,
  PetscScalar *color)
 {
   PetscScalar edge0, edge1, edge3;
   int whichplane, ierr=0;
+  struct isurface *thesurf = (struct isurface *) Surf;
 
   edge0 = (isoquant-vals[0]) / (vals[1]-vals[0]);
   edge1 = (isoquant-vals[1]) / (vals[2]-vals[1]);
@@ -345,7 +439,8 @@ int DrawTet
 			     coords[3],coords[4],coords[5],vals[1],
 			     coords[6],coords[7],coords[8],vals[2],
 			     coords[9],coords[10],coords[11],vals[3],
-			     isoquant, edge0,edge1,edge3, whichplane, color);
+			     isoquant, edge0,edge1,edge3, whichplane, color,
+			     thesurf);
   return ierr;
 }
 
@@ -360,7 +455,10 @@ int DrawTet
 
   int DrawHex Returns 0 or an error code.
 
-  PetscScalar *coords Coordinates of hexahedron corner points: xmin, xmax, ymin, etc.
+  ISurface Surf ISurface object into which to draw this hexahedron's isoquant.
+
+  PetscScalar *coords Coordinates of hexahedron corner points: xmin, xmax,
+  ymin, etc.
 
   PetscScalar *vals Function values at hexahedron corners: f0, f1, f2, etc.
 
@@ -370,13 +468,14 @@ int DrawTet
   ++++++++++++++++++++++++++++++++++++++*/
 
 int DrawHex
-(PetscScalar *coords, PetscScalar *vals, PetscScalar isoquant,
+(ISurface Surf, PetscScalar *coords, PetscScalar *vals, PetscScalar isoquant,
  PetscScalar *color)
 {
   int tetras[6][4] = {{0,1,2,4}, {1,2,4,5}, {2,4,5,6}, {1,3,2,5}, {2,5,3,6}, {3,6,5,7}};
   int tet,node,ierr;
   int c0,c1,c2,c3,whichplane;
   PetscScalar edge0,edge1,edge3;
+  struct isurface *thesurf = (struct isurface *) Surf;
 
   for(tet=0; tet<6; tet++)
     {
@@ -398,7 +497,8 @@ int DrawHex
 	     coords[c1&1],coords[2+((c1&2)>>1)],coords[4+((c1&4)>>2)],vals[c1],
 	     coords[c2&1],coords[2+((c2&2)>>1)],coords[4+((c2&4)>>2)],vals[c2],
 	     coords[c3&1],coords[2+((c3&2)>>1)],coords[4+((c3&4)>>2)],vals[c3],
-	     isoquant, edge0,edge1,edge3, whichplane, color); CHKERRQ (ierr);
+	     isoquant, edge0,edge1,edge3, whichplane, color, thesurf);
+	  CHKERRQ (ierr);
 	}
     }
 
@@ -410,12 +510,14 @@ int DrawHex
 #define __FUNCT__ "Draw3DBlock"
 
 /*++++++++++++++++++++++++++++++++++++++
-  Calculate vertices of isoquant triangle(s) in a 3-D array of right regular
-  hexahedra.  This loops through a 3-D array and calls DrawHex to calculate the
-  triangulation of each hexahedral cell.
+  Calculate vertices of isoquant triangle(s) in a 3-D regular array of right
+  regular hexahedra.  This loops through a 3-D array and calls DrawHex to
+  calculate the triangulation of each hexahedral cell.
 
   int Draw3DBlock Returns 0 or an error code.
 
+  ISurface Surf ISurface object into which to draw this block's isoquants.
+
   int xd Overall x-width of function value array.
 
   int yd Overall y-width of function value array.
@@ -434,7 +536,8 @@ int DrawHex
 
   int zm Z-width of the array section we'd like to draw.
 
-  PetscScalar *minmax Position of block corners: xmin, xmax, ymin, ymax, zmin, zmax.
+  PetscScalar *minmax Position of block corners: xmin, xmax, ymin, ymax, zmin,
+  zmax.
 
   PetscScalar *vals The array of function values at vertices.
 
@@ -448,7 +551,7 @@ int DrawHex
   ++++++++++++++++++++++++++++++++++++++*/
 
 int Draw3DBlock
-(int xd,int yd,int zd, int xs,int ys,int zs, int xm,int ym,int zm,
+(ISurface Surf,int xd,int yd,int zd, int xs,int ys,int zs,int xm,int ym,int zm,
  PetscScalar *minmax, PetscScalar *vals, int skip, int n_quants,
  PetscScalar *isoquants, PetscScalar *colors)
 {
@@ -483,8 +586,8 @@ int Draw3DBlock
 	      funcs[7] = vals[skip*(start+xd*yd+xd+1)];
 	      for (q=0; q<n_quants; q++)
 		{
-		  ierr = DrawHex (boxcoords, funcs, isoquants[q], colors+4*q);
-		  CHKERRQ (ierr);
+		  ierr = DrawHex (Surf, boxcoords, funcs, isoquants[q],
+				  colors+4*q); CHKERRQ (ierr);
 		}
 	    }
 	}
