@@ -1,11 +1,12 @@
 /***************************************
-  $Header: /cvsroot/petscgraphics/render.c,v 1.17 2005/04/28 09:14:36 hazelsct Exp $
+  $Header: /cvsroot/petscgraphics/render.c,v 1.35 2006/01/25 00:57:04 hazelsct Exp $
 
   This file contains the rendering code for Illuminator, which renders 2-D or
   3-D data into an RGB(A) unsigned char array (using perspective in 3-D).
 ***************************************/
 
 #include "illuminator.h"
+#include "surface.h"
 
 /* Build with -DDEBUG for debugging output */
 #undef DPRINTF
@@ -107,9 +108,51 @@ static inline void pseudoternarycolor
 	(A-scale[0])*(scale[3]-scale[1])) * inverse_det;
 
   /* Now colorize */
-  *pixel++ = 255*(1.-x1);
+  /* Simple R-G-B */
+  /* *pixel++ = 255*x1;
+  *pixel++ = 255*(1.-x1-x2);
+  *pixel++ = 255*sqrt(x2); */
+  /* *pixel++ = 255*(1.-x1);
   *pixel++ = 255*(x1+x2);
-  *pixel++ = 255*(1.-x2);
+  *pixel++ = 255*(1.-x2); */
+  /* Fancy three-square colorization */
+  if (2*x1+x2<=1 && 2*x2+x1<=1)
+    {
+      *pixel++ = 245;
+      *pixel++ = 245*(2*x1+3*x1*x2);
+      *pixel = 245*(2*x2+3*x1*x2);
+    }
+  else if (x2>=x1)
+    {
+      *pixel++ = 245*(2*(1-x1-x2)+3*(1-x1-x2)*x1);
+      *pixel++ = 245*(2*x1+3*(1-x1-x2)*x1);
+      *pixel = 245;
+    }
+  else
+    {
+      *pixel++ = 245*(2.-2*x1-2*x2+3*(1-x1-x2)*x2);
+      *pixel++ = 245;
+      *pixel = 245*(2*x2+3*(1-x1-x2)*x2);
+    }
+  /* Even fancier three-square colorization */
+  /*if (2*x1+x2<=1 && 4*x2+x1<=1) // 0,0 corner
+    {
+      *pixel++ = 254;
+      *pixel++ = 254*(2*x1+7./6.*x1*x2);
+      *pixel = 254*(4*x2+7./4.*x1*x2);
+    }
+  else if (3*x2>=x1) // 0,1 corner
+    {
+      *pixel++ = 254*(4./3.*(1-x1-x2)+7./3.*(1-x1-x2)*x1);
+      *pixel++ = 254*(4./3.*x1+7./3.*(1-x1-x2)*x1);
+      *pixel = 254;
+    }
+  else // 1,0 corner
+    {
+      *pixel++ = 254*(2*(1-x1-x2)+7./3.*(1-x1-x2)*x2);
+      *pixel++ = 254;
+      *pixel = 254*(4*x2+7./3.*(1-x1-x2)*x2);
+    }*/
 }
 
 
@@ -253,64 +296,36 @@ static inline void pseudoshearcolor
 
   int render_scale_2d It returns zero or an error code.
 
-  guchar *rgb Image to draw into.
-
-  int rwidth Intended width of the image.
-
-  int rheight Intended height of the image.
-
-  int bytes_per_pixel Number of bytes per pixel in the image.
+  IDisplay Disp Display object into which to draw the scale.
 
   field_plot_type fieldtype Type of plot.
 
   int symmetry Symmetry order for vector scale image.
   ++++++++++++++++++++++++++++++++++++++*/
 
-int render_scale_2d (guchar *rgb, int rwidth, int rheight, int bytes_per_pixel,
-		     field_plot_type fieldtype, int symmetry)
+int render_scale_2d (IDisplay Disp, field_plot_type fieldtype, int symmetry)
 {
   PetscScalar scale[6];
   int i, j, myheight;
+  struct idisplay *thedisp = (struct idisplay *) Disp;
 
   switch (fieldtype)
     {
     case FIELD_SCALAR:
       scale[0]=0.;
       scale[1]=1.;
-      for (j=0; j<rheight; j++)
-	for (i=0; i<rwidth; i++)
-	  pseudocolor ((PetscScalar) i/(rwidth-1), scale,
-		       rgb+(j*rwidth + i) * bytes_per_pixel);
-      return 0;
-
-    case FIELD_TERNARY:
-      scale[0] = scale[1] = scale[3] = scale[4] = 0.;
-      scale[2] = scale[5] = 1.;
-      myheight = ((PetscScalar) rheight > 0.5*sqrt(3.) * rwidth) ?
-	(int) (0.5*sqrt(3.)* rwidth) : rheight;
-      for (j=0; j<myheight; j++)
-	for (i=0; i<(int) (2./sqrt(3.)*(myheight-j)); i++)
-	  pseudoternarycolor
-	    ((PetscScalar) i/(myheight*2./sqrt(3.)),
-	     (PetscScalar) j/myheight, scale,
-	     rgb + (((rheight+myheight)/2 -j)*rwidth +i +(int)(sqrt(3.)/3.*j))*
-	     bytes_per_pixel);
-      return 0;
-
-    case FIELD_TERNARY_SQUARE:
-      scale[0] = scale[2] = 0.;
-      scale[1] = scale[3] = 1.;
-      for (j=0; j<rheight; j++)
-	for (i=0; i<rwidth; i++)
-	  pseudoternarysquarecolor
-	    ((PetscScalar) i/(rwidth-1), (PetscScalar) j/(rheight-1), scale,
-	     rgb+(j*rwidth + i) * bytes_per_pixel);
+      for (j=0; j<thedisp->rgb_height; j++)
+	for (i=0; i<thedisp->rgb_width; i++)
+	  pseudocolor ((PetscScalar) i/(thedisp->rgb_width-1), scale,
+		       thedisp->rgb+(j*thedisp->rgb_rowskip + i) *
+		       thedisp->rgb_bpp);
       return 0;
 
     case FIELD_VECTOR:
       scale[0] = 0.;
       scale[1] = 1.;
-      myheight = ((rheight > rwidth) ? rwidth : rheight)/2;
+      myheight = ((thedisp->rgb_height > thedisp->rgb_width) ?
+		  thedisp->rgb_width : thedisp->rgb_height)/2;
       for (j=-myheight; j<myheight; j++)
 	for (i=-(int)(myheight*sqrt(1.-((PetscScalar)j*j/myheight/myheight)));
 	     i<(int)(myheight*sqrt(1.-((PetscScalar)j*j/myheight/myheight)));
@@ -320,13 +335,41 @@ int render_scale_2d (guchar *rgb, int rwidth, int rheight, int bytes_per_pixel,
 	      theta = atan2 (j, i);
 	    pseudohueintcolor
 	      (mag * cos (symmetry*theta), mag * sin (symmetry*theta), scale,
-	       rgb + ((myheight-j)*rwidth + rwidth/2+i)*bytes_per_pixel);
+	       thedisp->rgb + ((myheight-j)*thedisp->rgb_rowskip +
+			       thedisp->rgb_width/2+i)*thedisp->rgb_bpp);
 	  }
       return 0;
 
+    case FIELD_TERNARY:
+      scale[0] = scale[1] = scale[3] = scale[4] = 0.;
+      scale[2] = scale[5] = 1.;
+      myheight = ((PetscScalar) thedisp->rgb_height >
+		  0.5*sqrt(3.) * thedisp->rgb_width) ?
+	(int) (0.5*sqrt(3.)* thedisp->rgb_width) : thedisp->rgb_height;
+      for (j=0; j<myheight; j++)
+	for (i=0; i<(int) (2./sqrt(3.)*(myheight-j)); i++)
+	  pseudoternarycolor
+	    ((PetscScalar) i/(myheight*2./sqrt(3.)),
+	     (PetscScalar) j/myheight, scale, thedisp->rgb +
+	     (((thedisp->rgb_height+myheight)/2 -j) * thedisp->rgb_rowskip +
+	      i + (int)(sqrt(3.)/3.*j)) * thedisp->rgb_bpp);
+      return 0;
+
+    case FIELD_TERNARY_SQUARE:
+      scale[0] = scale[2] = 0.;
+      scale[1] = scale[3] = 1.;
+      for (j=0; j<thedisp->rgb_height; j++)
+	for (i=0; i<thedisp->rgb_width; i++)
+	  pseudoternarysquarecolor
+	    ((PetscScalar) i/(thedisp->rgb_width-1),
+	     (PetscScalar) j/(thedisp->rgb_height-1), scale,
+	     thedisp->rgb+(j*thedisp->rgb_rowskip + i) * thedisp->rgb_bpp);
+      return 0;
+
     case FIELD_TENSOR_SHEAR:
       scale[0] = 1.;
-      myheight = ((rheight > rwidth) ? rwidth : rheight)/2;
+      myheight = ((thedisp->rgb_height > thedisp->rgb_width) ?
+		  thedisp->rgb_width : thedisp->rgb_height)/2;
       for (j=-myheight; j<myheight; j++)
 	for (i=-(int)(myheight*sqrt(1.-((PetscScalar)j*j/myheight/myheight)));
 	     i<(int)(myheight*sqrt(1.-((PetscScalar)j*j/myheight/myheight)));
@@ -336,7 +379,8 @@ int render_scale_2d (guchar *rgb, int rwidth, int rheight, int bytes_per_pixel,
 	      theta = atan2 (j, i);
 	    pseudoshearcolor
 	      (mag * cos (2*theta), mag * sin (2*theta), scale,
-	       rgb + ((myheight-j)*rwidth + rwidth/2+i)*bytes_per_pixel);
+	       thedisp->rgb + ((myheight-j)*thedisp->rgb_rowskip +
+			       thedisp->rgb_width/2+i)*thedisp->rgb_bpp);
 	  }
       return 0;
     }
@@ -344,6 +388,106 @@ int render_scale_2d (guchar *rgb, int rwidth, int rheight, int bytes_per_pixel,
 }
 
 
+#undef __FUNCT__
+#define __FUNCT__ "render_composition_path"
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Render a composition path, such as a diffusion path or phase diagram, onto an
+  IDisplay image.
+
+  int render_composition_path Returns zero or an error code.
+
+  IDisplay Disp IDisplay object into which to draw the path.
+
+  PetscScalar comp_array Array of compositions for drawing.
+
+  int gridpoints Number of gridpoints in the composition array.
+
+  int num_fields Total number of fields in the composition array.
+
+  field_plot_type fieldtype The type of this field.
+
+  PetscScalar *scale Array of minimum and maximum values to pass to the
+  various pseudocolor functions; if NULL, call auto_scale to determine those
+  values.
+
+  PetscScalar red Red color for composition path.
+
+  PetscScalar green Green color for composition path.
+
+  PetscScalar blue Blue color for composition path.
+
+  PetscScalar alpha Alpha channel for composition path.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+int render_composition_path
+(IDisplay Disp, PetscScalar *comp_array, int gridpoints, int num_fields,
+ field_plot_type fieldtype, PetscScalar *scale,
+ PetscScalar red,PetscScalar green,PetscScalar blue,PetscScalar alpha)
+{
+  int i, ierr;
+  PetscScalar local_scale[6];
+  struct idisplay *thedisp = (struct idisplay *) Disp;
+
+  /* Sanity checks */
+  if (!thedisp)
+    SETERRQ (PETSC_ERR_ARG_CORRUPT, "Null display object");
+  if (!(thedisp->rgb))
+    SETERRQ (PETSC_ERR_ARG_WRONGSTATE,
+	     "Display object has no RGB buffer to draw in");
+
+  /* Determine default min and max if none are provided */
+  if (scale == NULL)
+    {
+      scale = local_scale;
+      ierr = auto_scale (comp_array, gridpoints, num_fields, 0, fieldtype, 2,
+			 scale); CHKERRQ (ierr);
+    }
+
+  for (i=0; i<gridpoints; i++)
+    {
+      PetscScalar A=comp_array [i*num_fields], B=comp_array [i*num_fields+1];
+      int sx, sy;
+      guchar *spixel;
+
+      if (fieldtype == FIELD_TERNARY)
+	{
+	  int myheight = ((PetscScalar) thedisp->rgb_height >
+			  0.5*sqrt(3.)*thedisp->rgb_width) ?
+	    (int) (0.5*sqrt(3.)* thedisp->rgb_width) : thedisp->rgb_height;
+	  PetscScalar inverse_det =
+	    1./((scale[2]-scale[0])*(scale[5]-scale[1]) -
+		(scale[4]-scale[0])*(scale[3]-scale[1]));
+	  sy = myheight*(((B-scale[1])*(scale[2]-scale[0]) -
+			  (A-scale[0])*(scale[3]-scale[1])) *inverse_det);
+	  sx = thedisp->rgb_width* (((A-scale[0])*(scale[5]-scale[1]) -
+				     (B-scale[1])*(scale[4]-scale[0])) *inverse_det)+
+	    thedisp->rgb_width*sy/thedisp->rgb_height/2;
+	  sy = (thedisp->rgb_height+myheight)/2 - sy;
+	}
+      else if (fieldtype == FIELD_TERNARY_SQUARE)
+	{
+	  sx = thedisp->rgb_width * (A-scale[0])/(scale[1]-scale[0]);
+	  sy = thedisp->rgb_height*(B-scale[2])/(scale[3]-scale[2]);
+	}
+      else
+	sx=sy=-1; /* Don't draw diffusion path */
+
+      if (sx>=0 && sy>=0 && sx<thedisp->rgb_width &&
+	  sy<thedisp->rgb_height)
+	{
+	  spixel = thedisp->rgb +
+	    thedisp->rgb_bpp*(sy*thedisp->rgb_rowskip + sx);
+	  *spixel = (1.-alpha)* *spixel + red*alpha*255;
+	  *(spixel+1) = (1.-alpha)* *(spixel+1) + green*alpha*255;
+	  *(spixel+2) = (1.-alpha)* *(spixel+2) + blue*alpha*255;
+	}
+    }
+  return 0;
+}
+
+
 #undef __FUNCT__
 #define __FUNCT__ "render_rgb_local_2d"
 
@@ -354,16 +498,7 @@ int render_scale_2d (guchar *rgb, int rwidth, int rheight, int bytes_per_pixel,
 
   int render_rgb_local_2d Returns zero or an error code.
 
-  guchar *rgb RGB buffer, or location in the RGB buffer, in which to render.
-
-  int rwidth Total width of the section of the RGB buffer to draw.
-
-  int rheight Total height of the section of the RGB buffer to draw.
-
-  int rowskip Total width of the RGB buffer.
-
-  int bytes_per_pixel Number of bytes per pixel in this RGB buffer (typically 3
-  or 4).
+  IDisplay Disp Display object holding the RGB buffer and its characteristics.
 
   PetscScalar *global_array Local array of global vector data to render.
 
@@ -394,16 +529,37 @@ int render_scale_2d (guchar *rgb, int rwidth, int rheight, int bytes_per_pixel,
   int xm Width of the local part of the global vector.
 
   int ym Height of the local part of the global vector.
+
+  int transform Transformation flags.
+
+  IDisplay SDisp Display object in which to draw the diffusion path (optional,
+  ignored if NULL).
+
+  PetscScalar dpred Red color for diffusion path points (0-1).
+
+  PetscScalar dpgreen Green color for diffusion path points (0-1).
+
+  PetscScalar dpblue Blue color for diffusion path points (0-1).
+
+  PetscScalar dpalpha Alpha channel for diffusion path points (0-1).
   ++++++++++++++++++++++++++++++++++++++*/
 
 int render_rgb_local_2d
-(guchar *rgb, int rwidth, int rheight, int rowskip, int bytes_per_pixel,
- PetscScalar *global_array, int num_fields, int display_field,
+(IDisplay Disp, PetscScalar *global_array, int num_fields, int display_field,
  field_plot_type fieldtype, PetscScalar *scale, int nx,int ny, int xs,int ys,
- int xm,int ym)
+ int xm,int ym, int transform, IDisplay SDisp, PetscScalar dpred,
+ PetscScalar dpgreen, PetscScalar dpblue, PetscScalar dpalpha)
 {
-  int ix,iy, ierr;
+  int ix,iy, ierr=0;
   PetscScalar local_scale[6];
+  struct idisplay *thedisp = (struct idisplay *) Disp;
+
+  /* Sanity checks */
+  if (!thedisp)
+    SETERRQ (PETSC_ERR_ARG_CORRUPT, "Null display object");
+  if (!(thedisp->rgb))
+    SETERRQ (PETSC_ERR_ARG_WRONGSTATE,
+	     "Display object has no RGB buffer to draw in");
 
   /* Determine default min and max if none are provided */
   if (scale == NULL)
@@ -416,12 +572,28 @@ int render_rgb_local_2d
 	   scale[1], scale[2], scale[3], scale[4], scale[5]);
 
   /* Do the rendering (note switch in inner loop, gotta get rid of that) */
-  for (iy=rheight*ys/ny; iy<rheight*(ys+ym)/ny; iy++)
-    for (ix=rwidth*xs/nx; ix<rwidth*(xs+xm)/nx; ix++)
+  for (iy=thedisp->rgb_height*ys/ny; iy<thedisp->rgb_height*(ys+ym)/ny; iy++)
+    for (ix=thedisp->rgb_width*xs/nx; ix<thedisp->rgb_width*(xs+xm)/nx; ix++)
       {
-	int vecindex = (((rheight-iy-1)*ny/rheight)*nx +
-			ix*nx/rwidth)*num_fields + display_field;
-	guchar *pixel = rgb + bytes_per_pixel*(iy*rowskip + ix);
+	int localx, localy;
+	if (transform & ROTATE_LEFT)
+	  {
+	    localx = ((transform & FLIP_HORIZONTAL) ?
+		      iy : thedisp->rgb_height-iy-1) * nx/thedisp->rgb_height;
+	    localy = ((transform & FLIP_VERTICAL) ?
+		      ix : thedisp->rgb_width-ix-1) * ny/thedisp->rgb_width;
+	  }
+	else
+	  {
+	    localx = ((transform & FLIP_HORIZONTAL) ?
+		      thedisp->rgb_width-ix-1 : ix) * nx/thedisp->rgb_width;
+	    localy = ((transform & FLIP_VERTICAL) ?
+		      iy : thedisp->rgb_height-iy-1) * ny/thedisp->rgb_height;
+	  }
+
+	int vecindex = (localy*xm + localx)*num_fields + display_field;
+	guchar *pixel = thedisp->rgb +
+	  thedisp->rgb_bpp*(iy*thedisp->rgb_rowskip + ix);
 
 	switch (fieldtype)
 	  {
@@ -464,7 +636,16 @@ int render_rgb_local_2d
 	    SETERRQ (PETSC_ERR_ARG_OUTOFRANGE, "Field type not yet supported");
 	  }
       }
-  return 0;
+
+  DPRINTF ("Done main rendering, doing scale.\n",0);
+
+  /* (Optional) diffusion path */
+  if (SDisp)
+    ierr=render_composition_path (SDisp, global_array+display_field, xm*ym,
+				  num_fields, fieldtype, scale,
+				  dpred,dpgreen,dpblue,dpalpha);
+
+  return ierr;
 }
 
 
@@ -516,9 +697,9 @@ int render_rgb_local_2d
   +latex+Calling the matrix $M$ the inverse of the matrix on the left side of
   +latex+equation \ref{eq:matrix} gives the result:
   +latex+\begin{eqnarray}
-  +latex+  a&=& M_{00} (p_x-i_x) + M_{01} (p_y-i_y) + M_{02} (p_z-i_z), \\
-  +latex+  x&=& \frac{1}{a}[M_{10}(p_x-i_x)+M_{11}(p_y-i_y)+M_{12}(p_z-i_z)],\\
-  +latex+  y&=& \frac{1}{a}[M_{20}(p_x-i_x)+M_{21}(p_y-i_y)+M_{22}(p_z-i_z)].
+  +latex+  a&=& M_{00} (p_x-i_x) + M_{01} (p_y-i_y) + M_{02} (p_z-i_z),
+  +latex+  \\ x&=& \frac{1}{a}[M_{10}(p_x-i_x)+M_{11}(p_y-i_y)+M_{12}(p_z-i_z)],
+  +latex+  \\ y&=& \frac{1}{a}[M_{20}(p_x-i_x)+M_{21}(p_y-i_y)+M_{22}(p_z-i_z)].
   +latex+\end{eqnarray}
   +html+ This system can easily be solved for <i>x</i> and <i>y</i> by first
   +html+ making it into a matrix equation:
@@ -553,19 +734,10 @@ int render_rgb_local_2d
 
   int render_rgb_local_3d Returns zero or an error code.
 
-  guchar *rgb RGB buffer in which to render.
+  IDisplay Disp Display object holding the RGB buffer and its characteristics.
 
-  int rwidth Total width of the RGB buffer.
-
-  int rheight Total height of the RGB buffer.
-
-  int bytes_per_pixel Number of bytes per pixel in this RGB buffer (typically 3
-  or 4).
-
-  int num_triangles Number of triangles to render.
-
-  PetscScalar *vertices Table of coordinates (x1,y1,z1, x2,y2,z2, x3,y3,z3) and
-  colors (RGBA 0-1) making thirteen values per triangle.
+  ISurface Surf ISurface object containing triangles to render using imlib2
+  backend.
 
   PetscScalar *eye Point from where we're looking (x,y,z).
 
@@ -575,20 +747,29 @@ int render_rgb_local_2d
   ++++++++++++++++++++++++++++++++++++++*/
 
 int render_rgb_local_3d
-(guchar *rgb, int rwidth, int rheight, int bytes_per_pixel,
- int num_triangles, PetscScalar *vertices,
- PetscScalar *eye, PetscScalar *dir, PetscScalar *right)
+(IDisplay Disp, ISurface Surf, PetscScalar *eye, PetscScalar *dir,
+ PetscScalar *right)
 {
-#ifdef IMLIB2_EXISTS
   int *screen_coords, i;
   PetscScalar *shades, up[3], m00,m01,m02,m10,m11,m12,m20,m21,m22, a;
+  struct isurface *thesurf = (struct isurface *) Surf;
+  struct idisplay *thedisp = (struct idisplay *) Disp;
 
-  if (bytes_per_pixel != 4)
+  /* Sanity checks */
+  if (!thedisp)
+    SETERRQ (PETSC_ERR_ARG_CORRUPT, "Null display object");
+  if (!(thedisp->rgb))
+    SETERRQ (PETSC_ERR_ARG_WRONGSTATE,
+	     "Display object has no RGB buffer to draw in");
+
+  if (thedisp->rgb_bpp != 4)
     SETERRQ (PETSC_ERR_ARG_SIZ, "Imlib2 rendering requires 4 bytes per pixel");
 
   /* Allocate memory for the shades and integer coordinates */
-  i = PetscMalloc (6*num_triangles * sizeof(int), &screen_coords); CHKERRQ (i);
-  i = PetscMalloc (num_triangles * sizeof(PetscScalar), &shades); CHKERRQ (i);
+  i = PetscMalloc (6*thesurf->num_triangles * sizeof(int), &screen_coords);
+  CHKERRQ (i);
+  i = PetscMalloc (thesurf->num_triangles * sizeof(PetscScalar), &shades);
+  CHKERRQ (i);
 
   /* Calculate the "up" vector as described above, storing the magnitude of dir
      in a */
@@ -613,20 +794,20 @@ int render_rgb_local_3d
   m22 = (dir[0] *right[1] - dir[1]  *right[0]) / a;
 
   /* Loop over triangles, transforming their coordinates and setting shading */
-  for (i=0; i<num_triangles; i++)
+  for (i=0; i<thesurf->num_triangles; i++)
     {
       PetscScalar pmi00,pmi01,pmi02,pmi10,pmi11,pmi12,pmi20,pmi21,pmi22, c[3];
 
       /* Calculate coordinates relative to observer p-i (hence pmi) */
-      pmi00 = vertices[13*i]   - eye[0];
-      pmi01 = vertices[13*i+1] - eye[1];
-      pmi02 = vertices[13*i+2] - eye[2];
-      pmi10 = vertices[13*i+3] - eye[0];
-      pmi11 = vertices[13*i+4] - eye[1];
-      pmi12 = vertices[13*i+5] - eye[2];
-      pmi20 = vertices[13*i+6] - eye[0];
-      pmi21 = vertices[13*i+7] - eye[1];
-      pmi22 = vertices[13*i+8] - eye[2];
+      pmi00 = thesurf->vertices[13*i]   - eye[0];
+      pmi01 = thesurf->vertices[13*i+1] - eye[1];
+      pmi02 = thesurf->vertices[13*i+2] - eye[2];
+      pmi10 = thesurf->vertices[13*i+3] - eye[0];
+      pmi11 = thesurf->vertices[13*i+4] - eye[1];
+      pmi12 = thesurf->vertices[13*i+5] - eye[2];
+      pmi20 = thesurf->vertices[13*i+6] - eye[0];
+      pmi21 = thesurf->vertices[13*i+7] - eye[1];
+      pmi22 = thesurf->vertices[13*i+8] - eye[2];
 
       /* Calculate x,y for point 0 */
       a = m00*pmi00 + m01*pmi01 + m02*pmi02;
@@ -634,9 +815,9 @@ int render_rgb_local_3d
 	screen_coords [6*i+0] = screen_coords [6*i+1] = -1;
       else
 	{
-	  screen_coords [6*i+0] = rwidth/2 *
+	  screen_coords [6*i+0] = thedisp->rgb_width/2 *
 	    (1. + (m10*pmi00 + m11*pmi01 + m12*pmi02) / a);
-	  screen_coords [6*i+1] = rheight/2 - rwidth/2 *
+	  screen_coords [6*i+1] = thedisp->rgb_height/2 - thedisp->rgb_width/2 *
 	    (m20*pmi00 + m21*pmi01 + m22*pmi02) / a;
 	}
 
@@ -646,9 +827,9 @@ int render_rgb_local_3d
 	screen_coords [6*i+2] = screen_coords [6*i+3] = -1;
       else
 	{
-	  screen_coords [6*i+2] = rwidth/2 *
+	  screen_coords [6*i+2] = thedisp->rgb_width/2 *
 	    (1. + (m10*pmi10 + m11*pmi11 + m12*pmi12) / a);
-	  screen_coords [6*i+3] = rheight/2 - rwidth/2 *
+	  screen_coords [6*i+3] = thedisp->rgb_height/2 - thedisp->rgb_width/2 *
 	    (m20*pmi10 + m21*pmi11 + m22*pmi12) / a;
 	}
 
@@ -658,20 +839,21 @@ int render_rgb_local_3d
 	screen_coords [6*i+4] = screen_coords [6*i+5] = -1;
       else
 	{
-	  screen_coords [6*i+4] = rwidth/2 *
+	  screen_coords [6*i+4] = thedisp->rgb_width/2 *
 	    (1. + (m10*pmi20 + m11*pmi21 + m12*pmi22) / a);
-	  screen_coords [6*i+5] = rheight/2 - rwidth/2 *
+	  screen_coords [6*i+5] = thedisp->rgb_height/2 - thedisp->rgb_width/2 *
 	    (m20*pmi20 + m21*pmi21 + m22*pmi22) / a;
 	}
 
 #ifdef DEBUG
       if (i==0)
-	DPRINTF ("First triangle: (%g,%g,%g), (%g,%g,%g), (%g,%g,%g);\n"
-		 "2-D coordinates: (%d,%d), (%d,%d), (%d,%d)\n",
-		 vertices[0],vertices[1],vertices[2],vertices[3],vertices[4],
-		 vertices[5],vertices[6],vertices[7],vertices[8],
-		 screen_coords[0],screen_coords[1],screen_coords[3],
-		 screen_coords[3],screen_coords[4],screen_coords[5]);
+	DPRINTF("First triangle: (%g,%g,%g), (%g,%g,%g), (%g,%g,%g);\n"
+		"2-D coordinates: (%d,%d), (%d,%d), (%d,%d)\n",
+		thesurf->vertices[0],thesurf->vertices[1],thesurf->vertices[2],
+		thesurf->vertices[3],thesurf->vertices[4],thesurf->vertices[5],
+		thesurf->vertices[6],thesurf->vertices[7],thesurf->vertices[8],
+		screen_coords[0],screen_coords[1],screen_coords[3],
+		screen_coords[3],screen_coords[4],screen_coords[5]);
 #endif
 
       /* Calculate relative triangle centroid, triangle arms, and shade */
@@ -688,13 +870,28 @@ int render_rgb_local_3d
 	sqrt (pmi20*pmi20 + pmi21*pmi21 + pmi22*pmi22);
     }
 
+#ifdef IMLIB2_EXISTS
   /* Do the Imlib2 rendering */
-  imlib2_render_triangles ((DATA32 *) rgb, rwidth, rheight, num_triangles,
-			   screen_coords, vertices+9, 13, shades, 1);
+  imlib2_render_triangles
+    ((DATA32 *) thedisp->rgb, thedisp->rgb_width, thedisp->rgb_height, thesurf->num_triangles, screen_coords,
+     thesurf->vertices+9, 13, shades, 1);
 
-  i = PetscFree (shades); CHKERRQ (i);
-  i = PetscFree (screen_coords); CHKERRQ (i);
+  /* Imlib2 switches red and blue for some reason */
+  for (i=0; i<thedisp->rgb_height; i++)
+    {
+      int j;
+      for (j = i*thedisp->rgb_rowskip;
+	   j < i*thedisp->rgb_rowskip + thedisp->rgb_width; j++)
+	{
+	  guchar tmp = thedisp->rgb [4*j];
+	  thedisp->rgb[4*j] = thedisp->rgb[4*j+2];
+	  thedisp->rgb[4*j+2] = tmp;
+	}
+    }
 #else
   SETERRQ (PETSC_ERR_SUP, "3-D rendering requires Imlib2");
 #endif
+
+  i = PetscFree (shades); CHKERRQ (i);
+  i = PetscFree (screen_coords); CHKERRQ (i);
 }
