@@ -1,5 +1,5 @@
 /***************************************
-  $Header: /cvsroot/petscgraphics/tsview-ng.c,v 1.36 2005/04/28 09:15:16 hazelsct Exp $
+  $Header: /cvsroot/petscgraphics/tsview-ng.c,v 1.65 2006/02/05 21:27:20 hazelsct Exp $
 
   This program views the output of a time series saved using
   +latex+{\tt IlluMultiSave()}.
@@ -34,24 +34,38 @@ Then interactively flip through the timesteps (h or ? lists commands).\n";
 
 /*+ Declared in illuminator.c, these give the current number of triangles on
   this node and corner coordinates and color information for each triangle. +*/
-extern int num_triangles;
-extern PetscScalar vertices[];
 
 GladeXML *xml;
-/* Filename list */
+ISurface Surf;
+IDisplay Disp [1] = { NULL };
+
+/* Filename list and time/log info */
 int entrynum=0, total_entries=0, current_timestep;
 char *the_basename, *basedirname, **stepnames=NULL;
 double current_time;
+char *log_text=NULL;
+
+/* Ternary diffusion path color, and supplementary diffusion path data: there
+   are dp_supp_colors colors, each corresponding to a color and a number of
+   points in the big A and B arrays. */
+PetscScalar ternary_dp_color[4] = { 0.,0.,0.,1. };
+int dp_supp_colors=0, *dp_supp_color_points=NULL;
+PetscScalar *dp_supp_red=NULL, *dp_supp_green=NULL, *dp_supp_blue=NULL,
+  *dp_supp_alpha=NULL;
+PetscScalar *dp_supp_AB=NULL;
 
 /* Window parameters and drawables */
-int width=0, height=0, bpp=3, nx, ny, dataview_count=1;
+#define DEFAULT_RENDER_SIZE 300
+#define SCALE_WIDTH 200
+#define SCALE_HEIGHT 200
+#define SCALE_BPP 3
+int width=0, height=0, bpp=3, nx, ny, transform, dataview_count=1;
 GtkWidget *dataviews [1];
-guchar *rgbbuf [1] = { NULL }, scalar_rgb[120000], ternary_square_rgb[120000],
-  ternary_triangle_rgb[120000], *ternary_rgb, vector_rgb[120000],
-  shear_rgb[120000];
+IDisplay scalar_disp, ternary_square_disp, ternary_triangle_disp, ternary_disp,
+  vector_disp, shear_disp;
 gboolean scalar_auto_set=TRUE, ternary_auto_set=TRUE, ternary_square_set=FALSE,
   vector_auto_set=TRUE, shear_auto_set=TRUE;
-double sizemag;
+gdouble sizemag;
 PetscTruth transp;
 
 /* PETSc structures etc. */
@@ -68,7 +82,11 @@ int dimensions, num_fields, current_field, *field_index, num_variables[1],
 #undef __FUNCT__
 #define __FUNCT__ "render_dataviews"
 
-void render_dataviews ()
+/* Width and height are reversed if rotated, so use these macros with render */
+#define RENDER_WIDTH ((transform & ROTATE_LEFT) ? height : width)
+#define RENDER_HEIGHT ((transform & ROTATE_LEFT) ? width : height)
+
+int render_dataviews ()
 {
   int viewnum, nx,ny,nz, ierr;
 
@@ -86,19 +104,35 @@ void render_dataviews ()
       char thestatus [100];
 
       /* (Re)allocate buffer */
-      DPRINTF ("(Re)allocating RGB buffer\n",0);
-      if (!(rgbbuf [viewnum] =
-	    (guchar *) realloc (rgbbuf [viewnum],
-				bpp*width*height*sizeof(guchar)))) {
-	  printf ("ERROR: can't reallocate RGB buffer\n");
-	  exit (1); }
+      if (!Disp[0])
+	{
+	  DPRINTF ("Allocating IDisplay RGB buffer\n",0);
+	  if (IDispCreate (Disp, width, height, width, bpp, 0)) {
+	    printf ("ERROR: can't allocate RGB buffer\n");
+	    exit (1); }
+	}
+      else
+	{
+	  DPRINTF ("Reallocating IDisplay RGB buffer\n",0);
+	  if (IDispResize (Disp[0], width, height, width, bpp, 0)) {
+	    printf ("ERROR: can't reallocate RGB buffer\n");
+	    exit (1); }
+	}
+      DPRINTF ("Done, (re)sizing drawing area\n",0);
 
-      /* Render into rgbbuf [viewnum] */
+      /* Make sure the drawing area is the right size */
+      gtk_drawing_area_size (GTK_DRAWING_AREA (dataviews[viewnum]),
+			     width, height);
+
+      /* Render into Disp [viewnum] */
       ierr = DAGetInfo (theda, PETSC_NULL, &nx,&ny,PETSC_NULL,
 			PETSC_NULL,PETSC_NULL,PETSC_NULL, PETSC_NULL,
 			PETSC_NULL,PETSC_NULL,PETSC_NULL); CHKERRQ (ierr);
       ierr = DAGetCorners (theda, &xs,&ys,PETSC_NULL, &xm,&ym,PETSC_NULL);
       CHKERRQ (ierr);
+
+      DPRINTF ("Rendering %dx%d buffer with transform %d\n", RENDER_WIDTH,
+	       RENDER_HEIGHT, transform);
       if (dimensions == 2)
 	{
 	  char maxes [6][20];
@@ -123,15 +157,19 @@ void render_dataviews ()
 				      maxes[1]);
 		}
 	      ierr = render_rgb_local_2d
-		(rgbbuf [viewnum], width,height,width, bpp, global_array,
-		 num_fields, current_field, FIELD_SCALAR,
-		 scales, nx,ny, xs,ys, xm,ym); CHKERRQ (ierr);
+		(Disp[0], global_array, num_fields,current_field, FIELD_SCALAR,
+		 scales, nx,ny, xs,ys, xm,ym, transform, NULL,0,0,0,0);
+	      CHKERRQ (ierr);
 	      break;
 
 	    case FIELD_TERNARY:
 	    case FIELD_TERNARY_SQUARE:
 	      if (!ternary_square_set) /* Ternary triangle */
 		{
+		  GtkWidget *ternary_scale_area = glade_xml_get_widget
+		    (xml, "ternary_scale_area");
+		  int color, point, i;
+
 		  if (ternary_auto_set)
 		    {
 		      auto_scale (global_array, xm*ym, num_fields,
@@ -143,32 +181,51 @@ void render_dataviews ()
 		      snprintf (maxes[4], 19, "%g", scales[6]);
 		      snprintf (maxes[5], 19, "%g", scales[7]);
 		      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
-						     (xml, "ternary_1A_entry")),
+						     (xml,"ternary_1A_entry")),
 					  maxes[0]);
 		      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
-						     (xml, "ternary_1B_entry")),
+						     (xml,"ternary_1B_entry")),
 					  maxes[1]);
 		      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
-						     (xml, "ternary_2A_entry")),
+						     (xml,"ternary_2A_entry")),
 					  maxes[2]);
 		      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget 
-						     (xml, "ternary_2B_entry")),
+						     (xml,"ternary_2B_entry")),
 					  maxes[3]);
 		      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
-						     (xml, "ternary_3A_entry")),
+						     (xml,"ternary_3A_entry")),
 					  maxes[4]);
 		      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
-						     (xml, "ternary_3B_entry")),
+						     (xml,"ternary_3B_entry")),
 					  maxes[5]);
 		    }
+
+		  /* Start with blank ternary scale */
+		  render_scale_2d (ternary_triangle_disp, FIELD_TERNARY, 1);
+		  /* Overlay diffusion path supplements on scale */
+		  for (color=0, i=0; color<dp_supp_colors;
+		       i+=dp_supp_color_points[color++])
+		    render_composition_path
+		      (ternary_triangle_disp, dp_supp_AB+i*2,
+		       dp_supp_color_points[color], 2, FIELD_TERNARY, scales+2,
+		       dp_supp_red[color], dp_supp_green[color],
+		       dp_supp_blue[color], dp_supp_alpha[color]);
+		  /* Render diagram, add diffusion path to scale */
 		  ierr = render_rgb_local_2d
-		    (rgbbuf [viewnum], width,height,width, bpp, global_array,
-		     num_fields, current_field, FIELD_TERNARY, scales+2, nx,ny,
-		     xs,ys, xm,ym);
+		    (Disp[0], global_array, num_fields, current_field,
+		     FIELD_TERNARY, scales+2, nx,ny, xs,ys, xm,ym, transform,
+		     ternary_triangle_disp,
+		     ternary_dp_color[0],ternary_dp_color[1],
+		     ternary_dp_color[2],ternary_dp_color[3]);
 		  CHKERRQ (ierr);
+		  IDispDrawGdk (ternary_triangle_disp, ternary_scale_area,
+				GDK_RGB_DITHER_MAX);
 		}
 	      else /* Ternary square */
 		{
+		  GtkWidget *ternary_scale_area = glade_xml_get_widget
+		    (xml, "ternary_scale_area");
+
 		  if (ternary_auto_set)
 		    {
 		      auto_scale (global_array, xm*ym, num_fields,
@@ -179,23 +236,28 @@ void render_dataviews ()
 		      snprintf (maxes[2], 19, "%g", scales[10]);
 		      snprintf (maxes[3], 19, "%g", scales[11]);
 		      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
-						     (xml, "ternary_1A_entry")),
+						     (xml,"ternary_1A_entry")),
 					  maxes[0]);
 		      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
-						     (xml, "ternary_1B_entry")),
+						     (xml,"ternary_1B_entry")),
 					  maxes[1]);
 		      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
-						     (xml, "ternary_2A_entry")),
+						     (xml,"ternary_2A_entry")),
 					  maxes[2]);
 		      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget 
-						     (xml, "ternary_2B_entry")),
+						     (xml,"ternary_2B_entry")),
 					  maxes[3]);
 		    }
+
+		  render_scale_2d (ternary_triangle_disp, FIELD_TERNARY, 1);
 		  ierr = render_rgb_local_2d
-		    (rgbbuf [viewnum], width,height,width, bpp, global_array,
-		     num_fields, current_field, FIELD_TERNARY_SQUARE, scales+8,
-		     nx,ny, xs,ys, xm,ym);
-		  CHKERRQ (ierr);
+		    (Disp[0], global_array, num_fields, current_field,
+		     FIELD_TERNARY_SQUARE, scales+8, nx,ny, xs,ys, xm,ym,
+		     transform, ternary_square_disp,
+		     ternary_dp_color[0], ternary_dp_color[1],
+		     ternary_dp_color[2], ternary_dp_color[3]); CHKERRQ (ierr);
+		  IDispDrawGdk (ternary_square_disp, ternary_scale_area,
+				GDK_RGB_DITHER_MAX);
 		}
 	      break;
 
@@ -210,10 +272,9 @@ void render_dataviews ()
 				      maxes[0]);
 		}
 	      ierr = render_rgb_local_2d
-		(rgbbuf [viewnum], width,height,width, bpp, global_array,
-		 num_fields, current_field, FIELD_VECTOR, scales+12, nx,ny,
-		 xs,ys, xm,ym);
-	      CHKERRQ (ierr);
+		(Disp[0], global_array, num_fields, current_field,
+		 FIELD_VECTOR, scales+12, nx,ny, xs,ys, xm,ym, transform,
+		 NULL,0,0,0,0); CHKERRQ (ierr);
 	      break;
 
 	    case FIELD_TENSOR_SHEAR:
@@ -227,62 +288,42 @@ void render_dataviews ()
 				      maxes[0]);
 		}
 	      ierr = render_rgb_local_2d
-		(rgbbuf [viewnum], width,height,width, bpp, global_array,
-		 num_fields, current_field, FIELD_TENSOR_SHEAR, scales+14,
-		 nx,ny, xs,ys, xm,ym);
-	      CHKERRQ (ierr);
+		(Disp[0], global_array, num_fields, current_field,
+		 FIELD_TENSOR_SHEAR,scales+14, nx,ny, xs,ys, xm,ym, transform,
+		 NULL,0,0,0,0); CHKERRQ (ierr);
 	      break;
 	    }
 
 	  ierr = VecRestoreArray (global, &global_array); CHKERRQ (ierr);
 	}
-      else
+      else /* Three dimensions */
 	{
 	  /* Fixed negative z-direction for now */
 	  PetscScalar eye[3], dir[3] = {0, 0, -1.}, right[3] = {.5, 0., 0.};
+	  guchar bgcolor[4] = { 0x50, 0x50, 0x50, 0xFF };
 	  eye[0] = 0.5*(minmax[0]+minmax[1]);
 	  eye[1] = 0.5*(minmax[2]+minmax[3]);
 	  eye[2] = minmax[5] + minmax[1]-minmax[0];
 
-	  ierr = DATriangulate
-	    (theda, global, current_field, minmax, PETSC_DECIDE, PETSC_NULL,
-	     PETSC_NULL, PETSC_FALSE, PETSC_FALSE, PETSC_FALSE); CHKERRQ(ierr);
+	  if (!Surf)
+	    ierr = SurfCreate (&Surf); CHKERRQ (ierr);
 
-	  for (i=0; i<width*height; i++)
-	    {
-	      rgbbuf[viewnum][4*i] = 0x50;
-	      rgbbuf[viewnum][4*i+1] = 0x50;
-	      rgbbuf[viewnum][4*i+2] = 0x50;
-	      rgbbuf[viewnum][4*i+3] = 0xFF;
-	    }
-	  ierr = render_rgb_local_3d
-	    (rgbbuf [viewnum], width,height, bpp, num_triangles, vertices,
-	     eye, dir, right);
-	  for (i=0; i<width*height; i++)
-	    {
-	      guchar tmp = rgbbuf[viewnum][4*i];
-	      rgbbuf[viewnum][4*i] = rgbbuf[viewnum][4*i+2];
-	      rgbbuf[viewnum][4*i+2] = tmp;
-	    }
-	  num_triangles = 0;
+	  ierr = DATriangulate
+	    (Surf, theda, global, current_field, minmax, PETSC_DECIDE,
+	     PETSC_NULL, PETSC_NULL, PETSC_FALSE, PETSC_FALSE, PETSC_FALSE);
+	  CHKERRQ(ierr);
+
+	  IDispFill (Disp[0], bgcolor);
+	  ierr = render_rgb_local_3d (Disp, Surf, eye, dir, right);
+	  ierr = SurfClear (Surf); CHKERRQ (ierr);
 	}
 
-      /* Draw buffer onto window */
-      DPRINTF ("Painting rgb%s buffer onto window\n", (bpp==4) ? "_32" : "");
+      /* Draw IDisplay object onto window */
+      DPRINTF ("Painting %dx%d rgb%s buffer onto window\n",
+	       RENDER_WIDTH, RENDER_HEIGHT, (bpp==4) ? "_32" : "");
       if (!dataviews [viewnum])
 	dataviews [viewnum] = glade_xml_get_widget (xml, "plot_area");
-      gtk_drawing_area_size (GTK_DRAWING_AREA (dataviews [viewnum]),
-			     width, height);
-      if (bpp == 3)
-	gdk_draw_rgb_image (dataviews [viewnum]->window,
-			    dataviews[viewnum]->style->fg_gc[GTK_STATE_NORMAL],
-			    0, 0, width, height, GDK_RGB_DITHER_MAX,
-			    rgbbuf [viewnum], width * bpp);
-      else if (bpp == 4)
-	gdk_draw_rgb_32_image (dataviews [viewnum]->window,
-			    dataviews[viewnum]->style->fg_gc[GTK_STATE_NORMAL],
-			    0, 0, width, height, GDK_RGB_DITHER_MAX,
-			    rgbbuf [viewnum], width * bpp);
+      IDispDrawGdk (Disp[0], dataviews [viewnum], GDK_RGB_DITHER_MAX);
     }
 }
 
@@ -302,12 +343,12 @@ static char *basefilename;
   +latex+{\tt select()} function for {\tt scandir()} in {\tt main()}.
   +html+ <tt>select()</tt> function for <tt>scandir()</tt> in <tt>main()</tt>.
 
-  int myfilter Returns non-zero for qualifying filenames.
+  int my_time_filter Returns non-zero for qualifying filenames.
 
   const struct dirent *direntry Directory entry with filename to test.
   ++++++++++++++++++++++++++++++++++++++*/
 
-int myfilter (const struct dirent *direntry)
+int my_time_filter (const struct dirent *direntry)
 {
   if ((!strncmp (direntry->d_name, basefilename, strlen(basefilename))) &&
       (!strncmp (direntry->d_name + strlen(basefilename), ".time", 5)))
@@ -317,39 +358,119 @@ int myfilter (const struct dirent *direntry)
 }
 
 
-void on_plot_area_expose_event (GtkWidget *widget, GdkEventExpose *event,
-				gpointer user_data)
+/*++++++++++++++++++++++++++++++++++++++
+  This function returns non-zero for "qualifying" file names which start with
+  the stored files' basename and end with
+  +latex+{\tt .cpu0000.meta}.
+  +html+ <tt>.cpu0000.meta</tt>.
+  It is used as the
+  +latex+{\tt select()} function for {\tt scandir()} in {\tt main()}.
+  +html+ <tt>select()</tt> function for <tt>scandir()</tt> in <tt>main()</tt>.
+
+  int my_notime_filter Returns non-zero for qualifying filenames.
+
+  const struct dirent *direntry Directory entry with filename to test.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+int my_notime_filter (const struct dirent *direntry)
 {
-  if (bpp == 3)
-    gdk_draw_rgb_image (widget->window, widget->style->fg_gc[GTK_STATE_NORMAL],
-			0, 0, width, height, GDK_RGB_DITHER_MAX, rgbbuf [0],
-			width * bpp);
-  else if (bpp == 4)
-    gdk_draw_rgb_32_image (widget->window,
-			   widget->style->fg_gc[GTK_STATE_NORMAL],
-			   0, 0, width, height, GDK_RGB_DITHER_MAX, rgbbuf [0],
-			   width * bpp);
+  if ((!strncmp (direntry->d_name, basefilename, strlen(basefilename))))
+    return (!strncmp (direntry->d_name + strlen(direntry->d_name) - 13,
+		      ".cpu0000.meta", 13));
+  return 0;
 }
 
+
+/*++++++++++++++++++++++++++++++++++++++
+  This loads the names of the files into a long list.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+int refresh_stepnames ()
+{
+  struct dirent **namelist;
+  char *filec, *dirc, totalsteps[14];
+  int i, ierr;
+
+  filec = strdup (the_basename);
+  dirc = strdup (the_basename);
+  basefilename = basename (filec);
+  basedirname = dirname (dirc);
+
+  /* Default: scan for a timestep sequence */
+  total_entries = scandir (basedirname, &namelist, my_time_filter, alphasort);
+  if (total_entries < 0)
+    {
+      ierr = PetscPrintf (PETSC_COMM_WORLD, "Error scanning directory %s\n",
+			  basedirname); CHKERRQ (ierr);
+      return 1;
+    }
+  if (!total_entries)
+    {
+      /* Perhaps this is not a timestep sequence */
+      total_entries = scandir (basedirname, &namelist, my_notime_filter,
+			       alphasort);
+      if (total_entries < 0)
+	{
+	  ierr= PetscPrintf (PETSC_COMM_WORLD, "Error scanning directory %s\n",
+			      basedirname); CHKERRQ (ierr);
+	  return 1;
+	}
+      if (!total_entries)
+	{
+	  ierr = PetscPrintf (PETSC_COMM_WORLD, "No such files\n");
+	  CHKERRQ (ierr);
+	  return 1;
+	}
+    }
+  DPRINTF ("%d eligible files:\n", total_entries);
+  snprintf (totalsteps, 14, "/%d", total_entries-1);
+  gtk_label_set_text (GTK_LABEL (glade_xml_get_widget
+				 (xml, "total_label")), totalsteps);
+
+  if (!(stepnames = (char **) realloc
+	(stepnames, total_entries*sizeof (char *))))
+    {
+      ierr = PetscPrintf (PETSC_COMM_WORLD, "Error allocating memory\n");
+      CHKERRQ (ierr);
+      return 1;
+    }
+  for (i=0; i<total_entries; i++)
+    {
+      int filength = strlen(namelist[i]->d_name);
+
+      stepnames [i] = (char *) malloc ((filength-12)*sizeof(char));
+      strncpy (stepnames [i], namelist[i]->d_name, filength-13);
+      stepnames [i] [filength-13] = '\0';
+      free (namelist[i]);
+      DPRINTF ("[%d] %s\n", i, stepnames [i]);
+      if (ierr)
+	printf ("myfilter() Error: %d\n", ierr);
+      /* CHKERRQ (ierr); */
+    }
+
+  free (namelist);
+  return 0;
+}
+
+void on_plot_area_expose_event
+(GtkWidget *widget, GdkEventExpose *event, gpointer user_data)
+{ IDispDrawGdk (Disp[0], widget, GDK_RGB_DITHER_MAX); }
+
 void on_scalar_scale_area_expose_event
 (GtkWidget *widget, GdkEventExpose *event, gpointer user_data)
-{ gdk_draw_rgb_image (widget->window, widget->style->fg_gc[GTK_STATE_NORMAL],
-		      0, 0, 200, 200, GDK_RGB_DITHER_MAX, scalar_rgb, 600); }
+{ IDispDrawGdk (scalar_disp, widget, GDK_RGB_DITHER_MAX); }
 
 void on_ternary_scale_area_expose_event
 (GtkWidget *widget, GdkEventExpose *event, gpointer user_data)
-{ gdk_draw_rgb_image (widget->window, widget->style->fg_gc[GTK_STATE_NORMAL],
-		      0, 0, 200, 200, GDK_RGB_DITHER_MAX, ternary_rgb, 600); }
+{ IDispDrawGdk (ternary_disp, widget, GDK_RGB_DITHER_MAX); }
 
 void on_vector_scale_area_expose_event
 (GtkWidget *widget, GdkEventExpose *event, gpointer user_data)
-{ gdk_draw_rgb_image (widget->window, widget->style->fg_gc[GTK_STATE_NORMAL],
-		      0, 0, 200, 200, GDK_RGB_DITHER_MAX, vector_rgb, 600); }
+{ IDispDrawGdk (vector_disp, widget, GDK_RGB_DITHER_MAX); }
 
 void on_shear_scale_area_expose_event
 (GtkWidget *widget, GdkEventExpose *event, gpointer user_data)
-{ gdk_draw_rgb_image (widget->window, widget->style->fg_gc[GTK_STATE_NORMAL],
-		      0, 0, 200, 200, GDK_RGB_DITHER_MAX, shear_rgb, 600); }
+{ IDispDrawGdk (shear_disp, widget, GDK_RGB_DITHER_MAX); }
 
 void on_scalar_auto_checkbutton_toggled
 (GtkWidget *thebutton, gpointer user_data)
@@ -484,7 +605,7 @@ void on_ternary_square_checkbutton_toggled
       gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget 
 				     (xml, "ternary_2B_entry")), maxes[3]);
 
-      ternary_rgb = ternary_square_rgb;
+      ternary_disp = ternary_square_disp;
     }
   else
     {
@@ -493,13 +614,13 @@ void on_ternary_square_checkbutton_toggled
       gtk_widget_show (glade_xml_get_widget (xml, "ternary_3B_label"));
       gtk_widget_show (glade_xml_get_widget (xml, "ternary_3B_entry"));
       gtk_label_set_text (GTK_LABEL (glade_xml_get_widget
-				     (xml, "ternary_1A_label")), "Magenta A");
+				     (xml, "ternary_1A_label")), "Red A");
       gtk_label_set_text (GTK_LABEL (glade_xml_get_widget
-				     (xml, "ternary_1B_label")), "Magenta B");
+				     (xml, "ternary_1B_label")), "Red B");
       gtk_label_set_text (GTK_LABEL (glade_xml_get_widget
-				     (xml, "ternary_2A_label")), "Cyan A");
+				     (xml, "ternary_2A_label")), "Green A");
       gtk_label_set_text (GTK_LABEL (glade_xml_get_widget
-				     (xml, "ternary_2B_label")), "Cyan B");
+				     (xml, "ternary_2B_label")), "Green B");
       snprintf (maxes[0], 19, "%g", scales[2]);
       snprintf (maxes[1], 19, "%g", scales[3]);
       snprintf (maxes[2], 19, "%g", scales[4]);
@@ -519,7 +640,7 @@ void on_ternary_square_checkbutton_toggled
       gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
 				     (xml, "ternary_3B_entry")), maxes[5]);
 
-      ternary_rgb = ternary_triangle_rgb;
+      ternary_disp = ternary_triangle_disp;
     }
   on_ternary_scale_area_expose_event
     (glade_xml_get_widget (xml, "ternary_scale_area"), NULL, NULL);
@@ -607,6 +728,93 @@ void on_ternary_3B_entry_changed (GtkWidget *theentry, gpointer user_data)
 { G_CONST_RETURN gchar *entrytext = gtk_entry_get_text (GTK_ENTRY (theentry));
   sscanf (entrytext, "%lf", scales+7); }
 
+void on_ternary_dp_red_entry_changed (GtkWidget *theentry, gpointer user_data)
+{ G_CONST_RETURN gchar *entrytext = gtk_entry_get_text (GTK_ENTRY (theentry));
+  sscanf (entrytext, "%lf", ternary_dp_color); }
+
+void on_ternary_dp_green_entry_changed(GtkWidget *theentry, gpointer user_data)
+{ G_CONST_RETURN gchar *entrytext = gtk_entry_get_text (GTK_ENTRY (theentry));
+  sscanf (entrytext, "%lf", ternary_dp_color+1); }
+
+void on_ternary_dp_blue_entry_changed (GtkWidget *theentry, gpointer user_data)
+{ G_CONST_RETURN gchar *entrytext = gtk_entry_get_text (GTK_ENTRY (theentry));
+  sscanf (entrytext, "%lf", ternary_dp_color+2); }
+
+void on_ternary_dp_alpha_entry_changed(GtkWidget *theentry, gpointer user_data)
+{ G_CONST_RETURN gchar *entrytext = gtk_entry_get_text (GTK_ENTRY (theentry));
+  sscanf (entrytext, "%lf", ternary_dp_color+3); }
+
+void on_path_filename_entry_activate (GtkWidget *theentry, gpointer user) {
+  G_CONST_RETURN gchar *entrytext = gtk_entry_get_text (GTK_ENTRY (theentry));
+  FILE *path_file;
+  gchar linebuf[200];
+  int color=-1, point=0, point_array_size=0, ierr;
+  DPRINTF ("Filename entered: %s\n", entrytext);
+
+  if (!(path_file=fopen(entrytext, "r"))) {
+    ierr=PetscPrintf (PETSC_COMM_WORLD, "Unable to open: %s\n", entrytext);
+    CHKERRQ (ierr);
+    return; }
+
+  dp_supp_colors=0;
+  while (fgets (linebuf, 199, path_file))
+    {
+      if (linebuf[0] == 'c' || linebuf[0] == 'C')
+	{
+	  /* Expand the color buffers */
+	  dp_supp_colors++;
+	  dp_supp_color_points = realloc
+	    (dp_supp_color_points, dp_supp_colors*sizeof(int));
+	  dp_supp_red = realloc
+	    (dp_supp_red, dp_supp_colors*sizeof(PetscScalar));
+	  dp_supp_green = realloc
+	    (dp_supp_green, dp_supp_colors*sizeof(PetscScalar));
+	  dp_supp_blue = realloc
+	    (dp_supp_blue, dp_supp_colors*sizeof(PetscScalar));
+	  dp_supp_alpha = realloc
+	    (dp_supp_alpha, dp_supp_colors*sizeof(PetscScalar));
+
+	  /* Properties of the new color entry */
+	  color = dp_supp_colors-1;
+	  dp_supp_color_points [color] = 0;
+	  sscanf (linebuf, "%*s %lf %lf %lf %lf", dp_supp_red+color,
+		  dp_supp_green+color,dp_supp_blue+color,dp_supp_alpha+color);
+	}
+      if (((linebuf[0] >= '0' && linebuf[0] <= '9') || linebuf[0] == '.') &&
+	  color > -1)
+	{
+	  if (point >= point_array_size)
+	    dp_supp_AB = realloc
+	      (dp_supp_AB,
+	       (point_array_size=(point_array_size?point_array_size*2:100))*
+	       2*sizeof(PetscScalar));
+	  sscanf (linebuf, "%lf %lf", dp_supp_AB+2*point,
+		  dp_supp_AB+2*point+1);
+	  dp_supp_color_points[color]++;
+	  point++;
+	}
+    }
+  fclose (path_file);
+
+#ifdef DEBUG
+  ierr=PetscPrintf (PETSC_COMM_WORLD, "Loaded path supplement:\n");
+  CHKERRQ (ierr);
+  for (color=0, point=0; color<dp_supp_colors; color++)
+    {
+      ierr=PetscPrintf (PETSC_COMM_WORLD, "%d: %d points, color %g %g %g %g\n",
+			color, dp_supp_color_points[color], dp_supp_red[color],
+			dp_supp_green[color], dp_supp_blue[color],
+			dp_supp_alpha[color]);
+      CHKERRQ (ierr);
+      for (ierr=0; ierr<dp_supp_color_points[color]; ierr++, point++)
+	PetscPrintf (PETSC_COMM_WORLD, " %g %g\n", dp_supp_AB [point*2],
+		     dp_supp_AB [point*2+1]);
+    }
+#endif
+
+  render_dataviews();
+}
+
 void on_vector_max_entry_changed (GtkWidget *theentry, gpointer user_data)
 { G_CONST_RETURN gchar *entrytext = gtk_entry_get_text (GTK_ENTRY (theentry));
   sscanf (entrytext, "%lf", scales+13); }
@@ -617,7 +825,7 @@ void on_vector_symm_spinbutton_changed (GtkWidget *theentry,gpointer user_data)
   sscanf (entrytext, "%d", &symmetry);
   /* Because this sometimes sends events with outrageous symmetry numbers */
   if (symmetry <= 10) {
-    render_scale_2d (vector_rgb, 200, 200, 3, FIELD_VECTOR, symmetry);
+    render_scale_2d (vector_disp, FIELD_VECTOR, symmetry);
     on_vector_scale_area_expose_event
       (glade_xml_get_widget (xml, "vector_scale_area"), NULL, user_data); }}
 
@@ -625,68 +833,14 @@ void on_shear_max_entry_changed (GtkWidget *theentry, gpointer user_data)
 { G_CONST_RETURN gchar *entrytext = gtk_entry_get_text (GTK_ENTRY (theentry));
   sscanf (entrytext, "%lf", scales+14); }
 
-/*++++++++++++++++++++++++++++++++++++++
-  This loads the names of the files into a long list.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-int refresh_stepnames ()
-{
-  struct dirent **namelist;
-  char *filec, *dirc;
-  int i, ierr;
-
-  filec = strdup (the_basename);
-  dirc = strdup (the_basename);
-  basefilename = basename (filec);
-  basedirname = dirname (dirc);
-
-  total_entries = scandir (basedirname, &namelist, myfilter, alphasort);
-  if (!total_entries)
-    {
-      ierr = PetscPrintf (PETSC_COMM_WORLD, "No such files\n");
-      CHKERRQ (ierr);
-      return 1;
-    }
-  if (total_entries < 0)
-    {
-      ierr = PetscPrintf (PETSC_COMM_WORLD, "Error scanning directory %s\n",
-			  basedirname); CHKERRQ (ierr);
-      ierr = PetscFinalize (); CHKERRQ(ierr);
-      return 1;
-    }
-  DPRINTF ("%d eligible files:\n", total_entries);
-
-  if (!(stepnames = (char **) realloc
-	(stepnames, total_entries*sizeof (char *))))
-    {
-      ierr = PetscPrintf (PETSC_COMM_WORLD, "Error allocating memory\n");
-      CHKERRQ (ierr);
-      return 1;
-    }
-  for (i=0; i<total_entries; i++)
-    {
-      int filength = strlen(namelist[i]->d_name);
-
-      stepnames [i] = (char *) malloc ((filength-12)*sizeof(char));
-      strncpy (stepnames [i], namelist[i]->d_name, filength-13);
-      stepnames [i] [filength-13] = '\0';
-      free (namelist[i]);
-      DPRINTF ("[%d] %s\n", i, stepnames [i]);
-      CHKERRQ (ierr);
-    }
-
-  free (namelist);
-  return 0;
-}
-
 void change_variable (GtkWidget *widget, gpointer user_data)
 {
   current_field = GPOINTER_TO_INT (widget);
   DPRINTF ("Switching to variable %d\n", current_field);
-  render_dataviews();
   gtk_notebook_set_current_page
     (GTK_NOTEBOOK (glade_xml_get_widget (xml, "scale_notebook")),
      fieldtypes [current_field] >> 4);
+  render_dataviews();
 }
 
 void on_mag_spin_value_changed (GtkWidget *mag_spin, gpointer user_data) {
@@ -696,6 +850,11 @@ void on_mag_spin_value_changed (GtkWidget *mag_spin, gpointer user_data) {
   width = (int) (minmax [1] * sizemag);
   height = (int) (minmax [3] * sizemag);
 
+  if (width == 0)
+    width = DEFAULT_RENDER_SIZE;
+  if (height == 0)
+    height = DEFAULT_RENDER_SIZE;
+
   render_dataviews();
 }
 
@@ -727,8 +886,7 @@ void display_timestep (int usermetacount, char **usermetanames,
 void on_save_activate (GtkWidget *widget, gpointer user_data)
 {
   int i, ierr;
-  char **usermetanames, **usermetadata, filename [200], number[10];
-  FILE *outppm;
+  char filename [200], number[10];
 
   strncpy (filename, basedirname, 198);
   strcat (filename, "/");
@@ -738,19 +896,7 @@ void on_save_activate (GtkWidget *widget, gpointer user_data)
   strncat (filename, ".ppm", 198 - strlen (filename));
 
   DPRINTF ("Saving image with filename %s\n", filename);
-  if (!(outppm = fopen (filename, "w")))
-    {
-      PetscPrintf (PETSC_COMM_WORLD, "Error opening file %s\n", filename);
-      return;
-    }
-  fprintf (outppm, "P6\n%d %d\n255\n", width, height);
-  /* If more than 3 bytes per pixel, write just the first three (RGB) */
-  if (bpp == 3)
-    fwrite (rgbbuf [0], sizeof (guchar), 3*width*height, outppm);
-  else
-    for (i=0; i<width*height; i++)
-      fwrite (rgbbuf [0] + i*bpp, sizeof (guchar), 3, outppm);
-  fclose (outppm);
+  IDispWritePPM (Disp[0], filename);
 }
 
 
@@ -776,17 +922,169 @@ void on_timestep_spin_value_changed (GtkWidget *timestep_spin, gpointer user_dat
   strcat (filename, "/");
   strncat (filename, stepnames [entrynum], 198 - strlen (filename));
 
-  ierr = IlluMultiRead (theda, global, filename, &usermetacount,&usermetanames,
-			&usermetadata); CHKERRQ (ierr);
+  ierr = IlluMultiRead (PETSC_COMM_WORLD, theda, global, filename,
+			&usermetacount,&usermetanames, &usermetadata);
+  CHKERRQ (ierr);
 
   display_timestep (usermetacount, usermetanames, usermetadata);
 }
 
 
+void on_transform_activate (GtkWidget *widget, gpointer user_data)
+{
+  GtkWidget *timestep_spin = glade_xml_get_widget (xml, "timestep_spin"),
+    *flip_horiz = glade_xml_get_widget (xml, "flip_horiz"),
+    *flip_vertical = glade_xml_get_widget (xml, "flip_vertical"),
+    *rotate_left = glade_xml_get_widget (xml, "rotate_left");
+
+  transform =
+    (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (flip_horiz)) ?
+     FLIP_HORIZONTAL : 0) |
+    (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (flip_vertical)) ?
+     FLIP_VERTICAL : 0) |
+    (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (rotate_left)) ?
+     ROTATE_LEFT : 0);
+
+  render_dataviews ();
+}
+
+
+void on_save_all_activate (GtkWidget *none, gpointer user_data) {
+  GtkWidget *timestep_spin = glade_xml_get_widget (xml, "timestep_spin");
+  int i, temp = entrynum;
+
+  for (i=0; i<total_entries; i++)
+    {
+      gchar appbar_message [30];
+      gtk_spin_button_set_value (GTK_SPIN_BUTTON (timestep_spin), (gfloat) i);
+      on_timestep_spin_value_changed (timestep_spin, user_data);
+
+      sprintf (appbar_message, "Saving image %d/%d", i+1, total_entries);
+      gnome_appbar_set_status
+	(GNOME_APPBAR (glade_xml_get_widget (xml, "main_appbar")),
+	 appbar_message);
+      gnome_appbar_set_progress_percentage
+	(GNOME_APPBAR (glade_xml_get_widget (xml, "main_appbar")),
+	 (gfloat) (i+1)/total_entries);
+      while (gtk_events_pending ())
+	gtk_main_iteration ();
+
+      on_save_activate (timestep_spin, user_data);
+    }
+
+  gnome_appbar_set_progress_percentage
+    (GNOME_APPBAR (glade_xml_get_widget (xml, "main_appbar")), 0.);
+  gnome_appbar_refresh (GNOME_APPBAR(glade_xml_get_widget(xml,"main_appbar")));
+  gtk_spin_button_set_value (GTK_SPIN_BUTTON (timestep_spin), (gfloat) temp);
+  on_timestep_spin_value_changed (timestep_spin, user_data);
+}
+
+
+void on_save_all_scale_activate (GtkWidget *none, gpointer user_data) {
+  GtkWidget *timestep_spin = glade_xml_get_widget (xml, "timestep_spin");
+  int i,j, ierr, temp = entrynum;
+  char filename [200], number [10];
+  IDisplay scale_image;
+
+  if (!(scale_image =
+	(fieldtypes [current_field] == FIELD_SCALAR) ? scalar_disp :
+	((fieldtypes [current_field] == FIELD_VECTOR) ? vector_disp :
+	 ((fieldtypes [current_field] == FIELD_TENSOR_SHEAR) ? shear_disp :
+	  ((fieldtypes [current_field] == FIELD_TERNARY_SQUARE ||
+	    fieldtypes [current_field] == FIELD_TERNARY) ? ternary_disp :
+	   NULL)))))
+    return;
+
+  for (i=0; i<total_entries; i++)
+    {
+      gchar appbar_message [30];
+      gtk_spin_button_set_value (GTK_SPIN_BUTTON (timestep_spin), (gfloat) i);
+      on_timestep_spin_value_changed (timestep_spin, user_data);
+
+      sprintf (appbar_message, "Saving image %d/%d", i+1, total_entries);
+      gnome_appbar_set_status
+	(GNOME_APPBAR (glade_xml_get_widget (xml, "main_appbar")),
+	 appbar_message);
+      gnome_appbar_set_progress_percentage
+	(GNOME_APPBAR (glade_xml_get_widget (xml, "main_appbar")),
+	 (gfloat) (i+1)/total_entries);
+      while (gtk_events_pending ())
+	gtk_main_iteration ();
+
+      strncpy (filename, basedirname, 198);
+      strcat (filename, "/");
+      strncat (filename, stepnames [entrynum], 198 - strlen (filename));
+      snprintf (number, 9, "-s%d", current_field);
+      strncat (filename, number, 198 - strlen (filename));
+      strncat (filename, ".ppm", 198 - strlen (filename));
+
+      DPRINTF ("Saving image with filename %s\n", filename);
+      IDispWritePPM (scale_image, filename);
+    }
+
+  gnome_appbar_set_progress_percentage
+    (GNOME_APPBAR (glade_xml_get_widget (xml, "main_appbar")), 0.);
+  gnome_appbar_refresh (GNOME_APPBAR(glade_xml_get_widget(xml,"main_appbar")));
+  gtk_spin_button_set_value (GTK_SPIN_BUTTON (timestep_spin), (gfloat) temp);
+  on_timestep_spin_value_changed (timestep_spin, user_data);
+}
+
+
 void on_refresh_activate (GtkWidget *none, gpointer user_data) {
   if (refresh_stepnames ()) exit (1); }
 
 
+/*++++++++++++++++++++++++++++++++++++++
+  This reloads the .log file.
+
+  GtkWidget *none Empty GtkWidget (unusable because it's a menu item).
+
+  gpointer user_data Empty pointer.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void on_log_reload_button_clicked (GtkWidget *none, gpointer user_data)
+{
+  FILE *run_log;
+  unsigned char run_log_filename [300], run_log_buffer [300];
+  int log_size=0, nextchar=0;
+
+  strcpy ((char *) run_log_buffer, "Loading new log info");
+  gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (xml, "log_text_label")),
+		      (char *) run_log_buffer);
+
+  strncpy ((char *) run_log_filename, the_basename, 298-strlen(".log"));
+  strcat ((char *) run_log_filename, ".log");
+  DPRINTF ("Loading log file %s\n", run_log_filename);
+  if (!(run_log = fopen ((char *) run_log_filename, "r")))
+    {
+      printf ("Can't find log file %s\n", run_log_filename);
+      return;
+    }
+
+  /* There's probably a MUCH better way to slurp a file into a string... */
+  while (nextchar != EOF)
+    {
+      int i,j;
+      for (i=0; i<300 && nextchar != EOF; i++)
+	run_log_buffer[i] = nextchar = fgetc (run_log);
+      log_text = (char *) realloc
+	(log_text, (log_size += i) * sizeof (char));
+      for (j=0; j<i; j++)
+	log_text [log_size-i+j] = run_log_buffer [j];
+    }
+  log_text [log_size-1] = '\0';
+  fclose (run_log);
+  gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (xml, "log_text_label")),
+		      log_text);
+}
+
+
+void on_run_log_activate (GtkWidget *none, gpointer user_data) {
+  if (!log_text)
+    on_log_reload_button_clicked (none, user_data);
+  gtk_widget_show (glade_xml_get_widget (xml, "log_window")); }
+
+
 void on_about_activate (GtkWidget *none, gpointer user_data) {
   gtk_widget_show (glade_xml_get_widget (xml, "about")); }
 
@@ -835,9 +1133,12 @@ int main (int argc, char *argv[])
   ierr = PetscPrintf (PETSC_COMM_WORLD, "\n"); CHKERRQ (ierr);
 #endif
 
+  /* Initial settings */
   ierr = PetscOptionsHasName (PETSC_NULL, "-no_transparency", &transp);
   CHKERRQ (ierr);
   transp = !transp;
+  Surf = NULL;
+  transform = 0;
 
   /* Kludge alert!  Setting argc to avoid gnome_program_init errors;
      fix: use GNOME arguments instead of PETSc. */
@@ -862,15 +1163,18 @@ int main (int argc, char *argv[])
 
   DPRINTF ("Loading list of timestep names\n",0);
   if (refresh_stepnames ())
-    exit (1);
+    {
+      ierr = PetscFinalize (); CHKERRQ(ierr);
+      exit (1);
+    }
 
   DPRINTF ("Loading first timestep, creating distributed array\n",0);
   strncpy (filename, basedirname, 198);
   strcat (filename, "/");
   strncat (filename, stepnames [0], 198 - strlen (stepnames [0]));
-  ierr = IlluMultiLoad (filename, &theda, minmax+1,minmax+3,minmax+5,
-			&fieldtypes, &usermetacount, &usermetanames,
-			&usermetadata);
+  ierr = IlluMultiLoad
+    (PETSC_COMM_WORLD, filename, &theda, minmax+1,minmax+3,minmax+5,
+     &fieldtypes, &usermetacount, &usermetanames, &usermetadata);
   CHKERRQ (ierr);
 
   /* Usermetadata xwidth, ywidth, zwidth override minmax in case IlluMulti
@@ -896,7 +1200,12 @@ int main (int argc, char *argv[])
   else if (dimensions == 2)
     bpp = 3;
   else
-    bpp = 4;
+    {
+      bpp = 4;
+      gtk_widget_hide (glade_xml_get_widget (xml, "flip_horiz"));
+      gtk_widget_hide (glade_xml_get_widget (xml, "flip_vertical"));
+      gtk_widget_hide (glade_xml_get_widget (xml, "rotate_left"));
+    }
 
   /* Build menu of field variables */
   variable_options = glade_xml_get_widget (xml, "variable_menu");
@@ -929,12 +1238,17 @@ int main (int argc, char *argv[])
   gtk_widget_show (variable_options);
 
   /* Scale images */
-  render_scale_2d (scalar_rgb, 200, 200, 3, FIELD_SCALAR, 1);
-  render_scale_2d (ternary_triangle_rgb, 200, 200, 3, FIELD_TERNARY, 1);
-  render_scale_2d (ternary_square_rgb, 200, 200, 3, FIELD_TERNARY_SQUARE, 1);
-  ternary_rgb = ternary_triangle_rgb;
-  render_scale_2d (vector_rgb, 200, 200, 3, FIELD_VECTOR, 1);
-  render_scale_2d (shear_rgb, 200, 200, 3, FIELD_TENSOR_SHEAR, 1);
+  IDispCreate (&scalar_disp, SCALE_WIDTH,SCALE_HEIGHT,SCALE_WIDTH,SCALE_BPP,0);
+  IDispCreate (&vector_disp, SCALE_WIDTH,SCALE_HEIGHT,SCALE_WIDTH,SCALE_BPP,0);
+  IDispCreate (&shear_disp, SCALE_WIDTH,SCALE_HEIGHT,SCALE_WIDTH,SCALE_BPP,0);
+  IDispCreate (&ternary_triangle_disp, SCALE_WIDTH, SCALE_HEIGHT, SCALE_WIDTH,
+	       SCALE_BPP, 0);
+  IDispCreate (&ternary_square_disp, SCALE_WIDTH, SCALE_HEIGHT, SCALE_WIDTH,
+	       SCALE_BPP, 0);
+  ternary_disp = ternary_triangle_disp;
+  render_scale_2d (scalar_disp, FIELD_SCALAR, 1);
+  render_scale_2d (vector_disp, FIELD_VECTOR, 1);
+  render_scale_2d (shear_disp, FIELD_TENSOR_SHEAR, 1);
 
   gtk_notebook_set_current_page
     (GTK_NOTEBOOK (glade_xml_get_widget (xml, "scale_notebook")),
@@ -950,13 +1264,24 @@ int main (int argc, char *argv[])
     gtk_widget_show (main_window);
   }
 
+  /* Set initial magnification */
+  sizemag = DEFAULT_RENDER_SIZE/PetscMax(minmax[1],minmax[3]);
+  DPRINTF ("Max dimension is %g, setting magnification to %g\n",
+	   PetscMax(minmax[1],minmax[3]), sizemag);
+  gtk_spin_button_set_value
+    (GTK_SPIN_BUTTON (glade_xml_get_widget (xml, "mag_spin")), sizemag);
+
   DPRINTF ("Displaying first timestep\n",0);
   display_timestep (usermetacount, usermetanames, usermetadata);
 
   DPRINTF ("Running main loop\n",0);
   gtk_main();
 
-  DPRINTF ("Finalizing and exitting.\n",0);
+  DPRINTF ("Finalizing and exiting.\n",0);
+  if (Surf) {
+    ierr = ISurfDestroy (Surf); CHKERRQ (ierr); }
+  if (Disp[0]) {
+    ierr = IDispDestroy (Disp[0]); CHKERRQ (ierr); }
   ierr = PetscFinalize (); CHKERRQ(ierr);
   return 0;
 }
