@@ -1,5 +1,5 @@
 /***************************************
-  $Header: /cvsroot/petscgraphics/tsview.c,v 1.35 2004/07/02 20:50:41 hazelsct Exp $
+  $Header: /cvsroot/petscgraphics/tsview.c,v 1.43 2006/01/30 04:53:58 hazelsct Exp $
 
   This program views the output of a time series saved using
   +latex+{\tt IlluMultiSave()}.
@@ -23,6 +23,8 @@ Then interactively flip through the timesteps (h or ? lists commands).\n";
   ###           Jump to timestep ###\n\
   t             Toggle Geomview transparency (3-D only)\n\
   v             Change field displayed (3-D only)\n\
+  d             Dump geomview to picture (3-D only), creates basename-f%d.ppm\n\
+  a             Dump all timesteps to pictures (3-D only)\n\
   p [v1 v2 ...] Set contour values for plotting or \"auto\" (3-D only)\n\
   r             Reloads entries in a directory\n\
   s size        Set maximum dimension of PETSc viewer windows (2-D only)\n\
@@ -264,6 +266,8 @@ int main (int argc, char *argv[])
 	{ 1.,0.,0.,.5, 1.,1.,0.,.5, 0.,1.,0.,.5, 0.,1.,1.,.5, 0.,0.,1.,.5,
 	  1.,0.,1.,.5 };
       field_plot_type *fieldtypes;
+      ISurface Surf;
+      IDisplay Disp;
 
       /* Load the vector */
       strcpy (basis, basedirname);
@@ -273,8 +277,8 @@ int main (int argc, char *argv[])
 			  current_entry, basis);
       if (loaded)
 	{
-	  ierr = IlluMultiRead (theda, global, basis, &usermetacount,
-				&usermetanames, &usermetadata);
+	  ierr = IlluMultiRead (PETSC_COMM_WORLD, theda, global, basis,
+				&usermetacount, &usermetanames, &usermetadata);
 	  CHKERRQ (ierr);
 	}
       else
@@ -283,9 +287,10 @@ int main (int argc, char *argv[])
 	  display_field = 0;
 	  minmax [0] = minmax [2] = minmax [4] = 0.;
 	  minmax [1] = minmax [3] = minmax [5] = 1.;
-	  ierr = IlluMultiLoad (basis, &theda, minmax+1, minmax+3, minmax+5,
-				&fieldtypes, &usermetacount, &usermetanames,
-				&usermetadata); CHKERRQ (ierr);
+	  ierr = IlluMultiLoad
+	    (PETSC_COMM_WORLD, basis, &theda, minmax+1, minmax+3, minmax+5,
+	     &fieldtypes, &usermetacount, &usermetanames, &usermetadata);
+	  CHKERRQ (ierr);
 	  ierr = DAGetGlobalVector (theda, &global); CHKERRQ (ierr);
 	  loaded = PETSC_TRUE;
 
@@ -324,7 +329,13 @@ int main (int argc, char *argv[])
 	    }
 	  else
 	    {
-	      ierr = GeomviewBegin (PETSC_COMM_WORLD);
+#ifdef GEOMVIEW
+	      ierr = SurfCreate (&Surf); CHKERRQ (ierr);
+	      ierr = GeomviewBegin (PETSC_COMM_WORLD, &Disp); CHKERRQ (ierr);
+#else
+	      SETERRQ (PETSC_ERR_SUP,
+		       "Built without Geomview, which is needed for 3-D");
+#endif
 	    }
 	}
 
@@ -365,21 +376,26 @@ int main (int argc, char *argv[])
 	  DPRINTF ("Calculating triangle locations\n",0);
 	  if (plots)
 	    {
-	      ierr = DATriangulateRange (theda, global, display_field, minmax,
-					 plots, plot_vals, plot_colors,
-					 xmin,xmax, ymin,ymax, zmin,zmax);
-	    }
-	  else
-	    {
-	      ierr = DATriangulateRange (theda, global, display_field, minmax,
-					 PETSC_DECIDE, PETSC_NULL, PETSC_NULL,
+	      ierr = DATriangulateRange (Surf, theda, global, display_field,
+					 minmax, plots, plot_vals, plot_colors,
 					 xmin,xmax, ymin,ymax, zmin,zmax);
 	      CHKERRQ (ierr);
 	    }
+	  else
+	    {
+	      ierr = DATriangulateRange (Surf, theda, global, display_field,
+					 minmax, PETSC_DECIDE, PETSC_NULL,
+					 PETSC_NULL, xmin,xmax, ymin,ymax,
+					 zmin,zmax);
+	      CHKERRQ (ierr);
+	    }
+#ifdef GEOMVIEW
 	  DPRINTF ("Consolidating triangles on head node and visualizing\n",0);
 	  ierr = GeomviewDisplayTriangulation
-	    (PETSC_COMM_WORLD, minmax, fieldname, transp);
+	    (PETSC_COMM_WORLD, Surf, Disp, minmax, fieldname, transp);
 	  CHKERRQ (ierr);
+#endif
+	  ierr = SurfClear (Surf); CHKERRQ (ierr);
 	}
 
       /* Free user data */
@@ -412,7 +428,10 @@ int main (int argc, char *argv[])
 	      }
 	    else
 	      {
-		ierr = GeomviewEnd (PETSC_COMM_WORLD); CHKERRQ (ierr);
+#ifdef GEOMVIEW
+		ierr = GeomviewEnd (PETSC_COMM_WORLD, Disp); CHKERRQ (ierr);
+#endif
+		ierr = ISurfDestroy (Surf); CHKERRQ (ierr);
 	      }
 	    ierr = PetscFinalize(); CHKERRQ (ierr);
 	    return 0;
@@ -473,6 +492,81 @@ int main (int argc, char *argv[])
 	      display_field = (display_field+1) % fields;
 	    break;
 	  }
+#ifdef GEOMVIEW
+	case 'd':
+	case 'D':
+	  {
+	    char filename [200], number[10];
+
+	    if (dims<3)
+	      {
+		ierr=PetscPrintf (PETSC_COMM_WORLD,
+				  "The 'd' command is for 3-D only.\n");
+		CHKERRQ (ierr);
+		break;
+	      }
+
+	    strncpy (filename, basedirname, 198);
+	    strcat (filename, "/");
+	    strncat (filename, files [current_entry], 198 - strlen (filename));
+	    snprintf (number, 9, "-f%d", display_field);
+	    strncat (filename, number, 198 - strlen (filename));
+	    strncat (filename, ".ppm", 198 - strlen (filename));
+
+	    DPRINTF ("Saving image with filename %s\n", filename);
+	    IDispWritePPM (Disp, filename);
+	    break;
+	  }
+	case 'a':
+	case 'A':
+	  {
+	    char filename [200], number[10], *fieldname;
+	    int entry;
+
+	    if (dims<3)
+	      {
+		ierr=PetscPrintf (PETSC_COMM_WORLD,
+				  "The 'a' command is for 3-D only.\n");
+		CHKERRQ (ierr);
+		break;
+	      }
+
+	    for (entry=0; entry<total_entries; entry++)
+	      {
+		DPRINTF ("Loading   file %d/%d\r", entry, total_entries);
+		strcpy (basis, basedirname);
+		strcat (basis, "/");
+		strcat (basis, files[entry]);
+		ierr = IlluMultiRead (PETSC_COMM_WORLD, theda, global, basis,
+				      &usermetacount, &usermetanames,
+				      &usermetadata); CHKERRQ (ierr);
+
+		DPRINTF ("Rendering file %d/%d\r", entry, total_entries);
+		ierr = DATriangulateRange
+		  (Surf, theda, global, display_field, minmax,
+		   plots?plots:PETSC_DECIDE, plots?plot_vals:PETSC_NULL,
+		   plots?plot_colors:PETSC_NULL, xmin,xmax, ymin,ymax,
+		   zmin,zmax); CHKERRQ (ierr);
+		ierr = DAGetFieldName (theda, display_field, &fieldname);
+		CHKERRQ (ierr);
+		ierr = GeomviewDisplayTriangulation
+		  (PETSC_COMM_WORLD, Surf, Disp, minmax, fieldname, transp);
+		CHKERRQ (ierr);
+		ierr = SurfClear (Surf); CHKERRQ (ierr);
+
+		DPRINTF ("Dumping   file %d/%d\r", entry, total_entries);
+		strncpy (filename, basedirname, 198);
+		strcat (filename, "/");
+		strncat (filename, files [entry], 198 - strlen (filename));
+		snprintf (number, 9, "-f%d", display_field);
+		strncat (filename, number, 198 - strlen (filename));
+		strncat (filename, ".ppm", 198 - strlen (filename));
+		IDispWritePPM (Disp, filename);
+	      }
+	    DPRINTF ("\n");
+	    break;
+	  }
+#endif
 	case 'r':
 	case 'R':
 	  {
@@ -549,7 +643,7 @@ int main (int argc, char *argv[])
 	    if (dims<3)
 	      {
 		ierr=PetscPrintf (PETSC_COMM_WORLD,
-				  "The 'p' command is for 2-D only.\n");
+				  "The 'p' command is for 3-D only.\n");
 		CHKERRQ (ierr);
 		break;
 	      }
