@@ -1,5 +1,5 @@
 /***************************************
-  $Header: /cvsroot/petscgraphics/utility.c,v 1.5 2004/11/04 22:47:45 hazelsct Exp $
+  $Header: /cvsroot/petscgraphics/utility.c,v 1.6 2005/04/19 01:35:14 hazelsct Exp $
 
   This file contains small utility functions for various aspects of
   visualization and storage.
@@ -96,6 +96,23 @@ int auto_scale
 	scale [5] = maxxpy - scale [0];
 	return 0;
       }
+    case FIELD_TERNARY_SQUARE:
+      {
+	scale [0] = scale [1] = global_array [display_field];
+	scale [2] = scale [3] = global_array [display_field+1];
+	for (i=1; i<points; i++)
+	  {
+	    scale [0] = PetscMin
+	      (scale [0], global_array [i*num_fields + display_field]);
+	    scale [1] = PetscMax
+	      (scale [1], global_array [i*num_fields + display_field]);
+	    scale [2] = PetscMin
+	      (scale [2], global_array [i*num_fields + display_field+1]);
+	    scale [3] = PetscMax
+	      (scale [3], global_array [i*num_fields + display_field+1]);
+	  }
+	return 0;
+      }
     case FIELD_VECTOR:
     case FIELD_VECTOR+1:
       scale++;
@@ -162,7 +179,8 @@ void field_indices (int nfields, int ds, field_plot_type *plottypes,
       if (plottypes [i] == FIELD_VECTOR ||
 	  plottypes [i] == FIELD_VECTOR+1)
 	i += ds-1;
-      else if (plottypes [i] == FIELD_TERNARY)
+      else if (plottypes [i] == FIELD_TERNARY ||
+	       plottypes [i] == FIELD_TERNARY_SQUARE)
 	i += 1;
       else if (plottypes [i] == FIELD_TENSOR_FULL)
 	i += ds*ds-1;
